/******************************************************************************
*
* Copyright 2006-2015 Freescale Semiconductor, Inc.
* Copyright 2016-2017 NXP
*
***************************************************************************//*!
*
* @file     ADC_MEAS_FLT.c
*
* @author   r63172
*
* @version  1.0.1.0
*
* @date     Jul-13-2012
*
* @brief    MPC5643L rev. 2.0 Board Support Package - ADC module
*
*******************************************************************************
*
* This file includes all functions and settings specific for MPC5643L_CB ver.2.0.
*
******************************************************************************/

/******************************************************************************
| Includes
-----------------------------------------------------------------------------*/
#include "ADC_MEAS_FLT.h"

/******************************************************************************
| External declarations
-----------------------------------------------------------------------------*/

/******************************************************************************
| Defines and macros            (scope: module-local)
-----------------------------------------------------------------------------*/

/******************************************************************************
| Typedefs and structures       (scope: module-local)
-----------------------------------------------------------------------------*/

/******************************************************************************
| Global variable definitions   (scope: module-exported)
-----------------------------------------------------------------------------*/

/******************************************************************************
| Global variable definitions   (scope: module-local)
-----------------------------------------------------------------------------*/

/******************************************************************************
| Function prototypes           (scope: module-local)
-----------------------------------------------------------------------------*/

/******************************************************************************
| Function implementations      (scope: module-local)
-----------------------------------------------------------------------------*/

/******************************************************************************
| Function implementations      (scope: module-exported)
-----------------------------------------------------------------------------*/

/**************************************************************************//*!
@brief      ADC software initialization routine.

@param[in,out]  *ptr    Pointer to structure of ADC module variables and
                        parameters

@return     # true - ADC initialization successful
            # false - ADC initialization error

@details    This function performs software ADC module initialization.

@note       ADC module as refered to in this function is not actual peripheral
            module but an instance of software defined module of structure
            type holding all necessary ADC parameters and variables.

@warning
******************************************************************************/
tBool ADC_ModuleClear(pmsmDrive_t *ptr)
{
    ptr->adc.measured.fltPhA.filt   = 0.0F;
    ptr->adc.measured.fltPhA.raw    = 0.0F;
    ptr->adc.measured.fltPhB.filt   = 0.0F;
    ptr->adc.measured.fltPhB.raw    = 0.0F;
    ptr->adc.measured.fltPhC.filt   = 0.0F;
    ptr->adc.measured.fltPhC.raw    = 0.0F;
    ptr->adc.measured.fltUdcb.filt  = 0.0F;
    ptr->adc.measured.fltUdcb.raw   = 0.0F;
    ptr->adc.measured.fltTemp.filt  = 0.0F;
    ptr->adc.measured.fltTemp.raw   = 0.0F;
    ptr->adc.measured.fltIdcb.filt  = 0.0F;
    ptr->adc.measured.fltIdcb.raw   = 0.0F;

    ptr->adc.flag.R             	= 0x0;

    ptr->adc.param.u16CalibSamples  = 0;
    ptr->adc.flag.B.calibInitDone   = 0;
    ptr->adc.flag.B.calibDone       = 0;

    return 1;
}

/**************************************************************************//*!
@brief      ADC software calibration routine.

@param[in,out]  *ptr    Pointer to structure of ADC module variables and
                        parameters

@return     # true - when ADC calibration ended successfully
            # false - when ADC calibration is ongoing

@details    This function performs offset calibration during the calibration
            phase of the application. It is not intended to be executed when
            application is in run mode.

@note       ADC module as refereed to in this function is not actual peripheral
            module but an instance of software defined module of structure
            type holding all necessary ADC parameters and variables.

@warning
******************************************************************************/
tBool ADC_ModuleCalib(pmsmDrive_t *ptr)
{
	static tFloat fltFiltOutput;

	if (!(ptr->adc.flag.B.calibInitDone))
	{
		ptr->adc.calibCntr = 4096; // 2^(8 + 4)  order to accommodate settling time of the filter

		ptr->adc.measured.fltPhA.filt   = 0.0F;
		ptr->adc.measured.fltPhB.filt   = 0.0F;
		ptr->adc.measured.fltPhC.filt   = 0.0F;
		ptr->adc.measured.fltUdcb.filt  = 0.0F;

		ptr->adc.flag.B.calibDone       = 0;
		ptr->adc.flag.B.calibInitDone   = 1;
    }

    if (!(ptr->adc.flag.B.calibDone))
    {
    	/* --------------------------------------------------------------
         * Phase A - DC offset data filtering using MA recursive filter
         * ------------------------------------------------------------ */
        ptr->adc.measured.fltPhA.filt        = MLIB_Add(ptr->adc.measured.fltPhA.filt,ptr->adc.measured.fltPhA.raw);
        fltFiltOutput                        = MLIB_Div(ptr->adc.measured.fltPhA.filt,ptr->adc.param.u16CalibSamples);
        ptr->adc.measured.fltPhA.filt        = MLIB_Sub(ptr->adc.measured.fltPhA.filt,fltFiltOutput);
        ptr->adc.offset[ptr->svmSector].fltPhA   = fltFiltOutput;

        /* --------------------------------------------------------------
         * Phase B - DC offset data filtering using MA recursive filter
         * ------------------------------------------------------------ */
        ptr->adc.measured.fltPhB.filt        = MLIB_Add(ptr->adc.measured.fltPhB.filt,ptr->adc.measured.fltPhB.raw);
        fltFiltOutput                  	     = MLIB_Div(ptr->adc.measured.fltPhB.filt,ptr->adc.param.u16CalibSamples);
        ptr->adc.measured.fltPhB.filt        = MLIB_Sub(ptr->adc.measured.fltPhB.filt,fltFiltOutput);
        ptr->adc.offset[ptr->svmSector].fltPhB   = fltFiltOutput;

        /* --------------------------------------------------------------
         * Phase C - DC offset data filtering using MA recursive filter
         * ------------------------------------------------------------ */
        ptr->adc.measured.fltPhC.filt        = MLIB_Add(ptr->adc.measured.fltPhC.filt,ptr->adc.measured.fltPhC.raw);
        fltFiltOutput                  		 = MLIB_Div(ptr->adc.measured.fltPhC.filt,ptr->adc.param.u16CalibSamples);
        ptr->adc.measured.fltPhC.filt        = MLIB_Sub(ptr->adc.measured.fltPhC.filt,fltFiltOutput);
        ptr->adc.offset[ptr->svmSector].fltPhC   = fltFiltOutput;

        if ((--ptr->adc.calibCntr)<=0)
        {
        	ptr->adc.flag.B.calibDone        = 1;    // end of DC offset calibration
        }
    }

    return (ptr->adc.flag.B.calibDone);
}

/**************************************************************************//*!
@brief      ADC software 2-phase current measurement routine.

@param[in,out]  *ptr    Pointer to structure of ADC module variables and
                        parameters

@return     # true - when ADC measurement ended successfully
            # false - when ADC measurement is ongoing, or error occurred.

@details    This function performs measurement of two phase currents from two
            shunt resistors, which are selected based on active SVM vector. Third
            current is calculated from the two measured according to following
            equation:
            \f[
                i_A + i_B + i_C = 0
            \f]
            This approach can be used for current measurement up to full PWM
            duty-cycle.

@note

@warning
******************************************************************************/
tBool ADC_Measure2Ph(pmsmDrive_t *ptr)
{
	switch(ptr->svmSector)
	{
		case 2:
		case 3:
				{
                	ptr->adc.measured.fltPhA.raw    = (tFloat)((ioctl(ptr->uni3PeriphCfg.PhABaseAddr,ADC_READ_VALUE, ptr->uni3PeriphCfg.PhAChnlOffset)));
                	ptr->adc.measured.fltPhC.raw    = (tFloat)((ioctl(ptr->uni3PeriphCfg.PhCBaseAddr,ADC_READ_VALUE, ptr->uni3PeriphCfg.PhCChnlOffset)));
                	ptr->adc.measured.fltUdcb.raw   = (tFloat)((ioctl(ptr->uni3PeriphCfg.UdcBaseAddr,ADC_READ_VALUE, ptr->uni3PeriphCfg.UdcChnlOffset)));

                    ptr->iAbcFbck.fltArg1 = MLIB_Mul(MLIB_Sub(ptr->adc.offset[ptr->svmSector].fltPhA,ptr->adc.measured.fltPhA.raw),2);
                    ptr->iAbcFbck.fltArg3 = MLIB_Mul(MLIB_Sub(ptr->adc.offset[ptr->svmSector].fltPhC,ptr->adc.measured.fltPhC.raw),2);
                    ptr->iAbcFbck.fltArg2 = MLIB_Neg(MLIB_Sub(ptr->iAbcFbck.fltArg1,ptr->iAbcFbck.fltArg3));
                    return(1);
				}
		case 4:
		case 5:
                {
                    ptr->adc.measured.fltPhA.raw    = (tFloat)((ioctl(ptr->uni3PeriphCfg.PhABaseAddr,ADC_READ_VALUE, ptr->uni3PeriphCfg.PhAChnlOffset)));
                    ptr->adc.measured.fltPhB.raw    = (tFloat)((ioctl(ptr->uni3PeriphCfg.PhBBaseAddr,ADC_READ_VALUE, ptr->uni3PeriphCfg.PhBChnlOffset)));
                    ptr->adc.measured.fltUdcb.raw   = (tFloat)((ioctl(ptr->uni3PeriphCfg.UdcBaseAddr,ADC_READ_VALUE, ptr->uni3PeriphCfg.UdcChnlOffset)));

                    ptr->iAbcFbck.fltArg1 = MLIB_Mul(MLIB_Sub(ptr->adc.offset[ptr->svmSector].fltPhA,ptr->adc.measured.fltPhA.raw),2);
                    ptr->iAbcFbck.fltArg3 = MLIB_Mul(MLIB_Sub(ptr->adc.offset[ptr->svmSector].fltPhB,ptr->adc.measured.fltPhB.raw),2);
                    ptr->iAbcFbck.fltArg2 = MLIB_Neg(MLIB_Sub(ptr->iAbcFbck.fltArg1,ptr->iAbcFbck.fltArg2));

                    return(1);
                }
		case 1:
		case 6:
		default:
                {
                	ptr->adc.measured.fltPhB.raw    = (tFloat)((ioctl(ptr->uni3PeriphCfg.PhBBaseAddr,ADC_READ_VALUE, ptr->uni3PeriphCfg.PhBChnlOffset)));
                	ptr->adc.measured.fltPhC.raw    = (tFloat)((ioctl(ptr->uni3PeriphCfg.PhCBaseAddr,ADC_READ_VALUE, ptr->uni3PeriphCfg.PhCChnlOffset)));
                	ptr->adc.measured.fltUdcb.raw   = (tFloat)((ioctl(ptr->uni3PeriphCfg.UdcBaseAddr,ADC_READ_VALUE, ptr->uni3PeriphCfg.UdcChnlOffset)));

                    ptr->iAbcFbck.fltArg1 = MLIB_Mul(MLIB_Sub(ptr->adc.offset[ptr->svmSector].fltPhB,ptr->adc.measured.fltPhB.raw),2);
                    ptr->iAbcFbck.fltArg3 = MLIB_Mul(MLIB_Sub(ptr->adc.offset[ptr->svmSector].fltPhC,ptr->adc.measured.fltPhC.raw),2);
                    ptr->iAbcFbck.fltArg2 = MLIB_Neg(MLIB_Sub(ptr->iAbcFbck.fltArg2,ptr->iAbcFbck.fltArg3));

                    return(1);
                }
    }
}
/**************************************************************************//*!
@brief      ADC software 3-phase current measurement routine.

@param[in,out]  *ptr    Pointer to structure of ADC module variables and
                        parameters

@return     # true - when ADC measurement ended successfully
            # false - when ADC measurement is ongoing, or error occurred.

@details    This function performs measurement of three phase currents from
            all three shunt resistors. Because a non-zero length PWM pulse width
            is required for successful current sample, this approach can not
            be utilized up to full PWM dutycycle.

@note

@warning
******************************************************************************/
tBool ADC_Measure3Ph(pmsmDrive_t *ptr, tFloat Imax)
{
	ptr->adc.measured.fltPhA.raw    = MLIB_Mul(((tFloat)(ioctl(ptr->uni3PeriphCfg.PhABaseAddr,ADC_READ_VALUE, ptr->uni3PeriphCfg.PhAChnlOffset))/(tFloat)0x0FFF-0.5F),Imax);
	ptr->adc.measured.fltPhB.raw    = MLIB_Mul(((tFloat)(ioctl(ptr->uni3PeriphCfg.PhBBaseAddr,ADC_READ_VALUE, ptr->uni3PeriphCfg.PhBChnlOffset))/(tFloat)0x0FFF-0.5F),Imax);
	ptr->adc.measured.fltPhC.raw    = MLIB_Mul(((tFloat)(ioctl(ptr->uni3PeriphCfg.PhCBaseAddr,ADC_READ_VALUE, ptr->uni3PeriphCfg.PhCChnlOffset))/(tFloat)0x0FFF-0.5F),Imax);

	ptr->iAbcFbck.fltArg1 = MLIB_Mul(MLIB_Sub(ptr->adc.offset[ptr->svmSector].fltPhA, ptr->adc.measured.fltPhA.raw),2);
	ptr->iAbcFbck.fltArg2 = MLIB_Mul(MLIB_Sub(ptr->adc.offset[ptr->svmSector].fltPhB, ptr->adc.measured.fltPhB.raw),2);
	ptr->iAbcFbck.fltArg3 = MLIB_Mul(MLIB_Sub(ptr->adc.offset[ptr->svmSector].fltPhC, ptr->adc.measured.fltPhC.raw),2);

	return(1);
}

/**************************************************************************//*!
@brief      ADC software 3-phase current measurement routine.

@param[in,out]  *ptr    Pointer to structure of ADC module variables and
                        parameters

@return     # true - when ADC measurement ended successfully
            # false - when ADC measurement is ongoing, or error occurred.

@details    This function performs measurement of three phase currents from
            all three shunt resistors. Because a non-zero length PWM pulse width
            is required for successful current sample, this approach can not
            be utilized up to full PWM dutycycle.

@note

@warning
******************************************************************************/
tBool ADC_UdcVoltageMeasure(pmsmDrive_t *ptr, tFloat Umax)
{
	ptr->adc.measured.fltUdcb.raw   = MLIB_Mul(MLIB_Div((tFloat)(ioctl(ptr->uni3PeriphCfg.UdcBaseAddr,ADC_READ_VALUE,ptr->uni3PeriphCfg.UdcChnlOffset)),(tFloat)0x0FFF),Umax);
	ptr->adc.measured.fltUdcb.filt  = GDFLIB_FilterIIR1(ptr->adc.measured.fltUdcb.raw,&ptr->uDcbFilter);

	return(1);
}

/* End of file */
