/******************************************************************************
*
* Copyright 2006-2015 Freescale Semiconductor, Inc.
* Copyright 2016-2017 NXP
*
***************************************************************************//*!
*
* @file     MC33905_routines.c
*
* @author   B15651
* 
* @version  1.0.1.0
* 
* @date     Jun-15-2011
* 
* @brief    MC33937 MOSFET pre-driver Init function file, with routines for
*           setup and diagnostic.
*
*******************************************************************************
*
* Detailed Description of the file.
*
******************************************************************************/

/*****************************************************************************
* External objects
******************************************************************************/
#include "MPC5643L_qs.h"
#include "MPC5643L_LLD_qs.h"
#include "MC33905_routines.h"

// transfer status
tBool       trans_status;

// SBC MC33905 wake-up source
tU16    sbcStat;

MC33905_REG_T   sbc_reg;


/**************************************************************************//*!
@brief          MC33905 read SBC flags

@param[in,out]  MC33905_REG_T *ptr
@param[in]      void

@return         void

@details        The function read flags from SBC device

                \image latex foc.pdf "My application" width=10cm

@note

@warning
******************************************************************************/
tBool mc33905ReadFlag(MC33905_REG_T *ptr)
{
    static  volatile tU16   sbcFlags;

    /* Read and Clear VREG flags */
    trans_status            = dspiSendWord(&ptr->dspiSet, SBC_RD_FLAG_REG_H, &sbcFlags);
    ptr->flags.regFlg.R     = ((sbcFlags & 0xff) << 8);

    trans_status            = dspiSendWord(&ptr->dspiSet, SBC_RD_FLAG_REG_L, &sbcFlags);
    ptr->flags.regFlg.R     = ptr->flags.regFlg.R | (sbcFlags & 0xff);

    /* Read and Clear CAN flags */
    trans_status            = dspiSendWord(&ptr->dspiSet, SBC_RD_FLAG_CAN_L, &sbcFlags);
    ptr->flags.canFlg.R     = sbcFlags & 0xff;

    trans_status            = dspiSendWord(&ptr->dspiSet, SBC_RD_FLAG_CAN_H, &sbcFlags);
    ptr->flags.canFlg.R     = ptr->flags.canFlg.R | ((sbcFlags & 0xff) << 8);

    /* Read and Clear IO flags */
    trans_status            = dspiSendWord(&ptr->dspiSet, SBC_RD_FLAG_IO_L, &sbcFlags);
    ptr->flags.ioFlg.R      = sbcFlags & 0xff;

    trans_status            = dspiSendWord(&ptr->dspiSet, SBC_RD_FLAG_IO_H, &sbcFlags);
    ptr->flags.ioFlg.R      = ptr->flags.ioFlg.R | ((sbcFlags & 0xff) << 8);

    /* Read and Clear INT flags */
    trans_status            = dspiSendWord(&ptr->dspiSet, SBC_RD_FLAG_INT_L, &sbcFlags);
    ptr->flags.intFlg.R    = (sbcFlags & 0xff);

    /* Read and Clear INT flags */
    trans_status            = dspiSendWord(&ptr->dspiSet, SBC_RD_FLAG_INT_H, &sbcFlags);
    ptr->flags.intFlg.R      = ptr->flags.intFlg.R | ((sbcFlags & 0xff) << 8);

    /* Read and Clear LIN1 flags */
    trans_status            = dspiSendWord(&ptr->dspiSet, SBC_RD_FLAG_LIN1_L, &sbcFlags);
    ptr->flags.lin1Flg.R    = (sbcFlags & 0xff);

    /* Read and Clear LIN2 flags */
    trans_status            = dspiSendWord(&ptr->dspiSet, SBC_RD_FLAG_LIN2_L, &sbcFlags);
    ptr->flags.lin2Flg.R    = (sbcFlags & 0xff);

    return(1);
}

/**************************************************************************//*!
@brief          The function read MC33905 SBC I/O real time and device
                identification bits.

@param[in,out]  MC33905_REG_T *ptr
@param[in]      void

@return         void

@details        The function read I/O real time and device identification value
                from SBC device.

                \image latex foc.pdf "My application" width=10cm

@note

@warning
******************************************************************************/
tBool mc33905ReadRtDevId(MC33905_REG_T *ptr)
{
    static  volatile tU16   sbcFlags;

    /* Read CAN status register value */
    trans_status            = dspiSendWord(&ptr->dspiSet, SBC_GET_CAN_STATUS, &sbcFlags);
    ptr->flags.canStat.R    = (sbcFlags & 0xff);

    /* Read LIN1 status register value */
    trans_status            = dspiSendWord(&ptr->dspiSet, SBC_GET_LIN1_STATUS, &sbcFlags);
    ptr->flags.lin1Stat.R   = (sbcFlags & 0xff);

    /* Read LIN2 status register value */
    trans_status            = dspiSendWord(&ptr->dspiSet, SBC_GET_LIN2_STATUS, &sbcFlags);
    ptr->flags.lin2Stat.R   = (sbcFlags & 0xff);

    return(1);
}

/**************************************************************************//*!
@brief          Resets the SBC watchdog

@param[in,out]  DSPI_SET_T *ptr
@param[in]      void

@return         void

@details        The function resets the SBC watchdog

                \image latex foc.pdf "My application" width=10cm

@note

@warning
******************************************************************************/
void mc33905ClearWdt(DSPI_SET_T *ptr)
{
    static  volatile tU16   sbcFlags;

    /* Clear watchdog */
    trans_status    = dspiSendWord(ptr, SBC_SET_WDG_REG_VALUE, &sbcFlags);
}

/**************************************************************************//*!
@brief          Sets the SBC watchdog period

@param[in,out]  DSPI_SET_T *ptr
@param[in]      u8WDT_Period, Code for desired period as defined in
                MC33905_routines.h(SBC_WDOG_2_5ms, SBC_WDOG_3ms, ..., SBC_WDOG_512ms)

@return         void

@details        The function sets the SBC watchdog period

                \image latex foc.pdf "My application" width=10cm

@note

@warning
******************************************************************************/
void mc33905SetWdtPeriod(MC33905_REG_T *ptr)
{
    static  volatile tU16   sbcFlags, dspiDataOut;

    dspiDataOut     = ptr->timA.R; //(SBC_SET_INITWDG_REG_VALUE | u8WDT_Period);
    trans_status    = dspiSendWord(&ptr->dspiSet, dspiDataOut, &sbcFlags);
}

/**************************************************************************//*!
@brief          SBC initial configuration with default values from MC33905.h

@param[in,out]  *ptr
@param[in]      void

@return         void

@details        The function configure SBC

                \image latex foc.pdf "My application" width=10cm

@note           

@warning        
******************************************************************************/
tBool mc33905Config(MC33905_REG_T *ptr)
{
    static      volatile tU16   u16SbcStatus,dspiDataOut;
    static      volatile tU32   PIT_MCR, PIT_LDVAL, PIT_TCTRL;

    /* Read 16 bit flags */
    ptr->dspiSet.txCmd      = (ASSERT_CS0 | ASSERT_CTAR1);

    /* Read and Clear all flags */
    mc33905ReadFlag(ptr);
    mc33905ReadRtDevId(ptr);

    /* Read and Clear SAFE flag */
    dspiDataOut             = SBC_GET_MODE_REG_H_FLAGS;
    trans_status            = dspiSendWord(&ptr->dspiSet, dspiDataOut, \
                              &u16SbcStatus);

    // Initialize device
    /* LIN1 and LIN2 */
    dspiDataOut             = ptr->initLinIo.R;
    trans_status            = dspiSendWord(&ptr->dspiSet, dspiDataOut, \
                              &u16SbcStatus);

    /* Watchdog settings */
    dspiDataOut             = ptr->initWdg.R;
    trans_status            = dspiSendWord(&ptr->dspiSet, dspiDataOut, \
                              &u16SbcStatus);

    /* VREG */
    dspiDataOut             = ptr->initReg.R;//(SBC_SET_INITREG_REG_VALUE | SBC_VREG_INITVAL);
    trans_status            = dspiSendWord(&ptr->dspiSet, dspiDataOut, \
                              &u16SbcStatus);

    /* MISC */
    dspiDataOut             = ptr->initMisc.R;
    trans_status            = dspiSendWord(&ptr->dspiSet, dspiDataOut, \
                              &u16SbcStatus);

    /* Wait at least 50 microseconds before writing to timers registers */
    // read orginal PIT channel setup
    PIT_MCR     = ioctl(PIT_BASE,PIT_READ_VALUE_REG_MCR, NULL);
    PIT_LDVAL   = ioctl(PIT_CHNL0_BASE,PIT_READ_VALUE_REG_LDVAL, NULL);
    PIT_TCTRL   = ioctl(PIT_CHNL0_BASE,PIT_READ_VALUE_REG_TCTRL, NULL);

    // wait defined time for power supply voltage stabilization
    delayNanoSec(50000);

    // write original PIT channel setup
    ioctl(PIT_BASE,PIT_WRITE_VALUE_REG_MCR, PIT_MCR);
    ioctl(PIT_CHNL0_BASE,PIT_SET_TMR_RELOAD_VALUE, PIT_LDVAL);
    ioctl(PIT_CHNL0_BASE,PIT_WRITE_VALUE_REG_TCTRL, PIT_TCTRL);

    /* Set watchdog default period */
    mc33905SetWdtPeriod(ptr);

    /* Transition to normal mode */
    mc33905ClearWdt(&ptr->dspiSet);

    /* Resources initial configuration */
    /* Interrupt sources */
    dspiDataOut             = ptr->interrupt.R;
    trans_status            = dspiSendWord(&ptr->dspiSet, dspiDataOut, \
                              &u16SbcStatus);

    /* Regulator */
    dspiDataOut             = ptr->reg.R;
    trans_status            = dspiSendWord(&ptr->dspiSet, dspiDataOut, \
                              &u16SbcStatus);


//    /* CAN transceiver */
//    dspiDataOut             = ptr->can.R;
//    trans_status            = dspiSendWord(&ptr->dspiSet, dspiDataOut, \
//                              &u16SbcStatus);

//    /* LIN transceivers */
//    dspiDataOut             = ptr->lin1.R;
//    trans_status            = dspiSendWord(&ptr->dspiSet, dspiDataOut, \
//                              &u16SbcStatus);
//    dspiDataOut             = ptr->lin2.R;
//    trans_status            = dspiSendWord(&ptr->dspiSet, dspiDataOut, \
//                              &u16SbcStatus);

    /* I/O configuration */
    dspiDataOut             = ptr->io.R;
    trans_status            = dspiSendWord(&ptr->dspiSet, dspiDataOut, \
                              &u16SbcStatus);

    /* MUX output */
    dspiDataOut             = ptr->mux.R;//SBC_SET_MUX_REG_VALUE, SBC_MUX_INITV);
    trans_status            = dspiSendWord(&ptr->dspiSet, dspiDataOut, \
                              &u16SbcStatus);

    /* Read MODE flag register to clear SAFE flag */
    dspiDataOut             = SBC_GET_MODE_REG_H_FLAGS;
    trans_status            = dspiSendWord(&ptr->dspiSet, dspiDataOut, \
                              &u16SbcStatus);

    // wait 10ms for power supply voltage stabilization
    // read orginal PIT channel setup
    PIT_MCR     = ioctl(PIT_BASE,PIT_READ_VALUE_REG_MCR, NULL);
    PIT_LDVAL   = ioctl(PIT_CHNL0_BASE,PIT_READ_VALUE_REG_LDVAL, NULL);
    PIT_TCTRL   = ioctl(PIT_CHNL0_BASE,PIT_READ_VALUE_REG_TCTRL, NULL);

    // wait defined time for power supply voltage stabilization
    delayNanoSec(DELAY_10MS);

    // write original PIT channel setup
    ioctl(PIT_BASE,PIT_WRITE_VALUE_REG_MCR, PIT_MCR);
    ioctl(PIT_CHNL0_BASE,PIT_SET_TMR_RELOAD_VALUE, PIT_LDVAL);
    ioctl(PIT_CHNL0_BASE,PIT_WRITE_VALUE_REG_TCTRL, PIT_TCTRL);

    return (trans_status);
}

/**************************************************************************//*!
@brief          CAN transceiver configuration

@param[in,out]  *ptr
@param[in]      void

@return         void

@details        The function configure SBC

                \image latex foc.pdf "My application" width=10cm

@note

@warning
******************************************************************************/
void mc33905CanCfg(MC33905_REG_T *ptr)
{
    static  volatile tU16   u16SbcStatus, dspiDataOut;
    /* CAN configuration */
    dspiDataOut             = ptr->can.R;
    trans_status            = dspiSendWord(&ptr->dspiSet, dspiDataOut, \
                              &u16SbcStatus);
}

///*******************************************************************************/
///**
//* \brief    Select cyclic sense period
//* \author   B05114
//* \param    u8CycSens_Period: value for  cyclic sense period
//* \return   void
//*/
//void des_MC33905_SetCyclicSense(UINT8 u8CycSens_Period)
//{
//     /* Update timer register */
//    des_MC33905_Write(SBC_REGISTER|SBC_CYC_SENSE_INT,u8CycSens_Period);
//}
//
///*******************************************************************************/
///**
//* \brief    Sets the SBC Stop mode
//* \author   B05114
//* \param    u8Mode: Wake-up feature in stop mode (OR combination is valid)     \n
//            (SBC_STOP_FWU, SBC_STOP_CYCSEN, SBC_STOP_CYCINT, SBC_STOP_WDEN)
//* \return   void
//*/
//void des_MC33905_StopMode(UINT8 u8StopMode)
//{
//    /* Send Stop mode command */
//    des_MC33905_Write(SBC_REGISTER|SBC_MODE_RM, SBC_MODE_4|u8StopMode);
//}
//
///*******************************************************************************/
///**
//* \brief    Sets the SBC Sleep mode
//* \author   B05114
//* \param    u8Mode: Desired wake-up feature in sleep mode                    \n
//            (SBC_SLEEP, SBC_SLEEP_CYCSEN, SBC_SLEEP_FWU, SBC_SLEEP_CYC_FWU)
//* \return   void
//*/
//void des_MC33905_SleepMode(UINT8 u8SleepMode)
//{
//    /* Send Sleep mode command */
//    des_MC33905_Write(SBC_REGISTER|SBC_MODE_RM, u8SleepMode);
//}
//
///*******************************************************************************/
//
