/******************************************************************************
*
* Copyright 2006-2015 Freescale Semiconductor, Inc.
* Copyright 2016-2017 NXP
*
***************************************************************************//*!
*
* @file     MC33937_routines.c
*
* @author   B15651
*
* @version  1.0.2.0
*
* @date     Jun-16-2011
*
* @brief    The MC33937 MOSFET pre-driver Init function file, with routines for
*           pre-driver setup and diagnostic.
*
*******************************************************************************
*
* Detailed Description of the file.
*
******************************************************************************/
 
/*! 
@if MC33937_GROUP
    @addtogroup MC33937_GROUP
@else
    @defgroup MC33937_GROUP   MC33937
@endif
*/ 


/******************************************************************************
| Includes
-----------------------------------------------------------------------------*/
#include "MPC5643L_qs.h"
#include "MC33937_routines.h"

/******************************************************************************
| Global variable definitions   (scope: module-local)
-----------------------------------------------------------------------------*/
    tBool   bFcnStatusMC33937;


/******************************************************************************
| Function implementations      (scope: module-exported)
-----------------------------------------------------------------------------*/

/**************************************************************************//*!
@ingroup        MC33937_GROUP
@brief          MC33937 pre-driver fault clear

@param[in,out]  *ptr

@return         void

@details        The function clears a portion of the fault latch corresponding \
                to MASK0 and MASK1 using lower four bits of commands. A 1 bit \
                clears the interrupt latch for that flag. INT remains asserted \
                if other unmasked faults are still present.

@note           

@warning
******************************************************************************/
void    mc33937ClearFault(MC33937_SET_T *ptr)
{
    volatile tU8 help;

    // Disable MC33937 for operation
    ioctl(SIUL_BASE,SIUL_CLEAR_PAD, ptr->enPcrNb);

    bFcnStatusMC33937   = dspiSendByte(&ptr->dspiSet, CLINT0_CMD, &help);
    bFcnStatusMC33937   = dspiSendByte(&ptr->dspiSet, CLINT1_CMD, &help);
    // Enable MC33937 for operation
    ioctl(SIUL_BASE,SIUL_SET_PAD, ptr->enPcrNb);
}


/**************************************************************************//*!
@ingroup        MC33937_GROUP
@brief          MC33937 pre-driver set Enable pin

@param[out]     void
@param[in]      void

@return         The MC33937_enable function returns nothing.

@details        The function sets enable inputs pin of MC33937 pre-driver ( \
                enable driver outputs). 


@note           This function is used to enable outputs of the MC33937

@warning
******************************************************************************/
void    mc33937Enable(MC33937_SET_T *ptr)
{
    // Enable MC33937 for operation
    ioctl(SIUL_BASE,SIUL_SET_PAD, ptr->enPcrNb);
}


/**************************************************************************//*!
@ingroup        MC33937_GROUP
@brief          Clear Enable pin of the MC33937 pre-driver.

@param[in,out]  *ptr

@return         void

@details        The function clear enable inputs of MC33937 pre-driver ( \
                disable driver outputs). The IC is in Standby mode.

@note           The IC is fully biased up and all functions are operating, the \
                output drivers actively turn off all of the external FETs \
                (after initialization). The SPI port is functional. Logic level\
                 outputs are driven with low impedance. SO is high impedance \
                unless CS is low. VDD, Charge Pump and VLS regulators are all \
                operating. The IC is ready to move to Enable Mode.

@warning
******************************************************************************/
void    mc33937Disable(MC33937_SET_T *ptr)
{
    // Disable MC33937 for operation
    ioctl(SIUL_BASE,SIUL_CLEAR_PAD, ptr->enPcrNb);
}

/**************************************************************************//*!
@ingroup        MC33937_GROUP
@brief          Reset the MC33937 pre-driver.

@param[in,out]  *ptr

@return         void

@details        The function resets MC33937 pre-driver

@note

@warning
******************************************************************************/
void    mc33937Reset(MC33937_SET_T *ptr)
{
    // Disable MC33937 for operation
    ioctl(SIUL_BASE,SIUL_CLEAR_PAD, ptr->rstPcrNb);
}

/**************************************************************************//*!
@ingroup        MC33937_GROUP
@brief          Read the status registers of the MC33937 pre-driver.

@param[in,out]  *SetPtr

@return         1

@details        The function read status registers values of the MC33937 pre-driver.
                The actual values of the status registers are stored in SetPtr
                structure.

@note           

@warning
******************************************************************************/
tBool   mc33937ReadSr(MC33937_SET_T *ptr)
{
    volatile tU8 help;

    // assert transmit command
    ptr->dspiSet.txCmd      = ASSERT_CS0 | ASSERT_CTAR0;
    // send NULL0 command - ignore return value (previous command is unknown)
    bFcnStatusMC33937       = dspiSendByte(&ptr->dspiSet, STATUS0_CMD, &help);

    // send NULL0 command - return value is status register 0 value
    bFcnStatusMC33937       = dspiSendByte(&ptr->dspiSet, STATUS1_CMD, &help);
    ptr->status.sr0.R    = help;
    // send NULL1 command - return value is status register 1 value
    bFcnStatusMC33937       = dspiSendByte(&ptr->dspiSet, STATUS2_CMD, &help);
    ptr->status.sr1.R    = help;
    // send NULL2 command - return value is status register 2 value
    bFcnStatusMC33937       = dspiSendByte(&ptr->dspiSet, STATUS3_CMD, &help);
    ptr->status.sr2.R    = help;
    // send NULL0 command - return value is status register 3 value
    bFcnStatusMC33937       = dspiSendByte(&ptr->dspiSet, STATUS0_CMD, &help);
    ptr->status.sr3      = help;

    return (1);
}


/**************************************************************************//*!
@ingroup        MC33937_GROUP
@brief          MC33937 pre-driver deadtime setup

@param[in,out]  *SetPtr

@return         bFcnStatus

@details        The function setup deadtime between top and bottom transistor
                of MC33937 pre-driver. Required deadtime value is stored in \
                structure item SetPtr->deadtime. The MC33937 real deadtime value \
                is stored in the SetPtr->Set_deadtime. The real deadtime value \
                corresponds to the value of the status register 3 multipied by \
                50ns.


@note           This function is included in the MC33937_config function

@warning
******************************************************************************/
tBool   mc33937DeadtimeSetup(MC33937_SET_T *ptr)
{
    static  volatile tU32       PIT_MCR, PIT_LDVAL, PIT_TCTRL;
    static  volatile tU32       deadtime_reg, pcr_regOrigSetup;
    static  volatile tU16       help1;
    static  volatile tU8        help_sr0;

    // send command for setup deadtime
    bFcnStatusMC33937   = dspiSendByte(&ptr->dspiSet, DEADTIME_CMD, &help_sr0);

    // read orginal PIT channel setup
    PIT_MCR     = ioctl(PIT_BASE,PIT_READ_VALUE_REG_MCR, NULL);
    PIT_LDVAL   = ioctl(ptr->pitChnlBaseAdr,PIT_READ_VALUE_REG_LDVAL, NULL);
    PIT_TCTRL   = ioctl(ptr->pitChnlBaseAdr,PIT_READ_VALUE_REG_TCTRL, NULL);

    delayNanoSec(20000);

    // configure pad as GPIO
    ioctl(SIUL_BASE,SIUL_CONFIGURE_PAD_GPIO, ptr->dspiSet.csPcrNb);

    // dead-time value is calculated (required_dead-time * 16) in [ns]
    // for the PIT timer
    deadtime_reg        = (tU32)(ptr->deadtime * MC33937_CALIB_CLK);

    // pull-down CS0 for setup deadtime
    ioctl(SIUL_BASE,SIUL_CLEAR_PAD, ptr->dspiSet.csPcrNb);

    delayNanoSec(deadtime_reg);

    // pull-up CS0 at the end of deadtime setup
    ioctl(SIUL_BASE,SIUL_SET_PAD, ptr->dspiSet.csPcrNb);

    // restore original pin setup
    ioctl(SIUL_BASE,SIUL_CONFIGURE_PAD_ALT1, ptr->dspiSet.csPcrNb);

    // read deadtime register from MC33937
    bFcnStatusMC33937       = dspiSendByte(&ptr->dspiSet, STATUS3_CMD, &help_sr0);
    bFcnStatusMC33937       = dspiSendByte(&ptr->dspiSet, STATUS2_CMD, &help_sr0);

    help1 = (tU16)(help_sr0);

    // return deadtime in [ns]
    ptr->setDeadtime        = ((tU16)(help_sr0) * 50);

    // return 1 if deadtime setup correctly
    return(1);
}


/**************************************************************************//*!
@ingroup        MC33937_GROUP
@brief          MC33937 pre-driver configuration function.

@param[in,out]  *SetPtr

@return

@details        The function provide configuration of MC33937 pre-driver.
  	  	  	    Function uses PIT module

@note           This function is used to configure MC33937
				Double RESET Timing Considerations (EB701 Rev.1.0/ 1/2009)

@warning
******************************************************************************/
void mc33937DoubleReset(MC33937_SET_T *ptr)
{
    static  volatile tU32       PIT_MCR, PIT_LDVAL, PIT_TCTRL;


    // EN1 and EN2 are still low -> VDD rise, init interrupt

    // read original PIT channel setup
    PIT_MCR     = ioctl(PIT_BASE,PIT_READ_VALUE_REG_MCR, NULL);
    PIT_LDVAL   = ioctl(PIT_CHNL0_BASE,PIT_READ_VALUE_REG_LDVAL, NULL);
    PIT_TCTRL   = ioctl(PIT_CHNL0_BASE,PIT_READ_VALUE_REG_TCTRL, NULL);

    // first sequence of the RESET
    ioctl(SIUL_BASE,SIUL_SET_PAD, ptr->rstPcrNb);

    // wait defined time for power supply voltage stabilization = 5ms
    delayNanoSec(MC33937_POWER_UP_TIME);

    //CLEAR first sequence of the RESET
    ioctl(SIUL_BASE,SIUL_CLEAR_PAD, ptr->rstPcrNb);

    // wait defined time for power supply voltage stabilization = 5ms
    delayNanoSec(MC33937_POWER_UP_TIME);

    // second sequence of the RESET
    ioctl(SIUL_BASE,SIUL_SET_PAD, ptr->rstPcrNb);

   // wait defined time for power supply voltage stabilization = 5ms
   delayNanoSec(MC33937_POWER_UP_TIME);

   // write original PIT channel setup
   ioctl(PIT_BASE,PIT_WRITE_VALUE_REG_MCR, PIT_MCR);
   ioctl(PIT_CHNL0_BASE,PIT_SET_TMR_RELOAD_VALUE, PIT_LDVAL);
   ioctl(PIT_CHNL0_BASE,PIT_WRITE_VALUE_REG_TCTRL, PIT_TCTRL);
}

/**************************************************************************//*!
@ingroup        MC33937_GROUP
@brief          MC33937 pre-driver configuration function.

@param[in,out]  *SetPtr

@return         dt_set_fault

@details        The function provides configuration of MC33937 dead-time
 	 	 	 	logic between top and bottom transistor.
 	 	 	 	Function uses DSPI module.

@note           This function is used to configure MC33937

@warning
******************************************************************************/
tBool mc33937Config(MC33937_SET_T *ptr)
{
    static  volatile tU16       deltaDT;
    static  volatile tU16       MC33937_deadError;
    static  volatile tU8        help_sr0, error_cnt;
    tBool                   dt_set_fault;

        // at first clear faults
    mc33937ClearFault(ptr);
    // Initialize MASK register to mask unwanted interrupts
    // send MASK0 and MASK1 command to mask out unwanted interrupts
    bFcnStatusMC33937   = dspiSendByte(&ptr->dspiSet, ptr->mask0, &help_sr0);
    bFcnStatusMC33937   = dspiSendByte(&ptr->dspiSet, ptr->mask1, &help_sr0);

    error_cnt                                   = 5;
    // setup desired dead-time of MC33937 value is in [ns]
    do {
        bFcnStatusMC33937   = mc33937DeadtimeSetup(ptr);
        deltaDT             = ptr->deadtime - ptr->setDeadtime;
        dt_set_fault        = (deltaDT > 150) ? 1 : 0;
        error_cnt--;
    // deadtime error is greather than 150ns, setup deadtime again
    }while(dt_set_fault && error_cnt);

    mc33937ClearFault(ptr);
    // send MODE command with desired bits, and also the LOCK bit to prevent further mode changes
    bFcnStatusMC33937   = dspiSendByte(&ptr->dspiSet, ptr->mode, &help_sr0);
    return (!dt_set_fault);
}
