/******************************************************************************
*
* Copyright 2006-2015 Freescale Semiconductor, Inc.
* Copyright 2016-2017 NXP
*
***************************************************************************//*!
*
* @file     MPC5643L_DMCB_BSP.h
*
* @author   r63172
*
* @version  1.0.1.0
*
* @date     2017
*
* @brief    Header file for MPC5643L Dual Motor Controller Board board support
*           package
*
*******************************************************************************
*
* This file includes MACRO definitions and settings specific for MPC5643L_DMCB
*
******************************************************************************/
#ifndef _MPC5643L_DMCB_BSP_H_
#define _MPC5643L_DMCB_BSP_H_

/******************************************************************************
* General definitions
******************************************************************************/


/***************************************************************************//*!
*
* @brief    Force all FlexPWM outputs to an OFF state with pre-defined output
*           polarity.
*
* @param    FLEXPWM_0   - FlexPWM 0 peripheral module
* @param    FLEXPWM_1   - FlexPWM 1 peripheral module   (not available on MPC560xP)
*
* @return   none
*
* @detail   Macro #SWITCH_PWM_OFF forces all pwm outputs of selected FlexPWM
*           module to an OFF (initial) state with pre-defined polarity.
*           PWM generators will keep running but output pins are forced
*           to a defined state.
*
* @note     Macro #SWITCH_PWM_OFF utilizes MASK register of FlexPWM module
*           therefore output pins state is no changed immediately but only after
*           FORCE_OUT signal (local reload).
*
******************************************************************************/
#define SWITCH_ALL_PWM_OFF(param)       ioctl(param,FLEXPWM_MASK,FLEXPWM_MASK_ENABLE_ALL);

/***************************************************************************//*!
*
* @brief    Enable normal operation of FlexPWM output logic circuitry.
*
* @param    FLEXPWM_0   - FlexPWM 0 peripheral module
* @param    FLEXPWM_1   - FlexPWM 1 peripheral module   (not available on MPC560xP)
*
* @return   none
*
* @detail   Macro #SWITCH_PWM_ON enables all pwm outputs of selected FlexPWM
*           module to an OFF (initial) state with pre-defined polarity.
*           PWM generators will keep running but output pins are forced
*           to a defined state.
*
* @note     Macro #SWITCH_PWM_OFF utilizes MASK register of FlexPWM module
*           therefore output pins state is no changed immediately but only after
*           FORCE_OUT signal (local reload).
*
******************************************************************************/
#define SWITCH_ALL_PWM_ON(param)        ioctl(param,FLEXPWM_MASK,FLEXPWM_MASK_DISABLE_ALL);

/***************************************************************************//*!
*
* @brief    Enable fault protection of FlexPWMA
*
* @param    FLEXPWM_0   - FlexPWM 0 peripheral module
* @param    FLEXPWM_1   - FlexPWM 1 peripheral module   (not available on MPC560xP)
*
* @return   none
*
* @detail   Macro #FAULT_PROTECT_PWMA_ON enables all PWMA outputs to be under
* 			fault protection shield.
*
* @note     Only for Submodules 0,1,2
*
******************************************************************************/
#define FAULT_PROTECT_PWMA_ON(param) \
		{\
		ioctl(param + FLEXPWM_SUB0_OFFSET,FLEXPWM_FAULT_MASK_PWMA,FLEXPWM_FAULTS_DISABLED);\
		ioctl(param + FLEXPWM_SUB1_OFFSET,FLEXPWM_FAULT_MASK_PWMA,FLEXPWM_FAULTS_DISABLED);\
		ioctl(param + FLEXPWM_SUB2_OFFSET,FLEXPWM_FAULT_MASK_PWMA,FLEXPWM_FAULTS_DISABLED);\
		}

/***************************************************************************//*!
*
* @brief    Enable fault protection of FlexPWMb
*
* @param    FLEXPWM_0   - FlexPWM 0 peripheral module
* @param    FLEXPWM_1   - FlexPWM 1 peripheral module   (not available on MPC560xP)
*
* @return   none
*
* @detail   Macro #FAULT_PROTECT_PWMA_ON enables all PWMb outputs to be under
* 			fault protection shield.
*
* @note		Only for Submodules 0,1,2
*
******************************************************************************/
#define FAULT_PROTECT_PWMB_ON(param) \
		{\
		ioctl(param + FLEXPWM_SUB0_OFFSET,FLEXPWM_FAULT_MASK_PWMB,FLEXPWM_FAULTS_DISABLED);\
		ioctl(param + FLEXPWM_SUB1_OFFSET,FLEXPWM_FAULT_MASK_PWMB,FLEXPWM_FAULTS_DISABLED);\
		ioctl(param + FLEXPWM_SUB2_OFFSET,FLEXPWM_FAULT_MASK_PWMB,FLEXPWM_FAULTS_DISABLED);\
		}

/***************************************************************************//*!
*
* @brief    Clear fault PIN flags manually
*
* @param    FLEXPWM_0   - FlexPWM 0 peripheral module
* @param    FLEXPWM_1   - FlexPWM 1 peripheral module   (not available on MPC560xP)
*
* @return   none
*
* @detail   Macro #CLEAR_FAULT_FLAGS_MAN provides clearing of all fault PIN flag
*
* @note
*
******************************************************************************/
#define CLEAR_FAULT_FLAGS_MAN(param) ioctl(param,FLEXPWM_CLEAR_ALL_FAULT_FLAGS,NULL);

/***************************************************************************//*!
*
* @brief    Get VAL1 number
*
* @param    FLEXPWM_0   - FlexPWM 0 peripheral module
* @param    FLEXPWM_1   - FlexPWM 1 peripheral module   (not available on MPC560xP)
*
* @return   none
*
* @detail   Macro #GET_FLEXPWM_VAL1_SUB0 gives a VAL1 value of Submodule0
*			Macro #GET_FLEXPWM_VAL1_SUB1 gives a VAL1 value of Submodule2
*			Macro #GET_FLEXPWM_VAL1_SUB2 gives a VAL1 value of Submodule2
*			Macro #GET_FLEXPWM_VAL1_SUB3 gives a VAL1 value of Submodule3
* @note
*
******************************************************************************/
#define GET_FLEXPWM_VAL1_SUB0(param) ioctl(param+FLEXPWM_SUB0_OFFSET,FLEXPWM_GET_VAL1, NULL);
#define GET_FLEXPWM_VAL1_SUB1(param) ioctl(param+FLEXPWM_SUB1_OFFSET,FLEXPWM_GET_VAL1, NULL);
#define GET_FLEXPWM_VAL1_SUB2(param) ioctl(param+FLEXPWM_SUB2_OFFSET,FLEXPWM_GET_VAL1, NULL);
#define GET_FLEXPWM_VAL1_SUB3(param) ioctl(param+FLEXPWM_SUB3_OFFSET,FLEXPWM_GET_VAL1, NULL);


/***************************************************************************//*!
*
* @brief    Get INIT number
*
* @param    FLEXPWM_0   - FlexPWM 0 peripheral module
* @param    FLEXPWM_1   - FlexPWM 1 peripheral module   (not available on MPC560xP)
*
* @return   none
*
* @detail   Macro #GET_FLEXPWM_INIT_SUB0 gives a INIT value of Submodule0
*			Macro #GET_FLEXPWM_INIT_SUB1 gives a INIT value of Submodule2
*			Macro #GET_FLEXPWM_INIT_SUB2 gives a INIT value of Submodule2
*			Macro #GET_FLEXPWM_INIT_SUB3 gives a INIT value of Submodule3
* @note
*
******************************************************************************/
#define GET_FLEXPWM_INIT_SUB0(param) ioctl(param+FLEXPWM_SUB0_OFFSET,FLEXPWM_GET_INIT, NULL);
#define GET_FLEXPWM_INIT_SUB1(param) ioctl(param+FLEXPWM_SUB1_OFFSET,FLEXPWM_GET_INIT, NULL);
#define GET_FLEXPWM_INIT_SUB2(param) ioctl(param+FLEXPWM_SUB2_OFFSET,FLEXPWM_GET_INIT, NULL);
#define GET_FLEXPWM_INIT_SUB3(param) ioctl(param+FLEXPWM_SUB3_OFFSET,FLEXPWM_GET_INIT, NULL);

/***************************************************************************//*!
*
* @brief    Update 3ph duty cycles for center aligned PWM
*
* @param    FLEXPWM_0   - FlexPWM 0 peripheral module
* @param    FLEXPWM_1   - FlexPWM 1 peripheral module   (not available on MPC560xP)
* @param    uint16_t	- 16bit value of dutycysle
*
* @return   none
*
* @detail   Macro #GET_FLEXPWM_VAL1_SUB0 gives a VAL1 value of Submodule0
*
* @note
*
******************************************************************************/
#define SET_3PH_CENTERAL_DUTYCYCLE(param, dutycycle) ioctl(param,FLEXPWM_SET_PWMA_DUTY_3PH_CENTERALIGN, dutycycle)

/***************************************************************************//*!
*
* @brief    Set LDOK for all submodules
*
* @param    FLEXPWM_0   - FlexPWM 0 peripheral module
* @param    FLEXPWM_1   - FlexPWM 1 peripheral module   (not available on MPC560xP)
*
* @return   none
*
* @detail   Macro #GET_FLEXPWM_VAL1_SUB0 gives a VAL1 value of Submodule0
*
* @note
*
******************************************************************************/
#define SET_PWM_LDOK(param) ioctl(param,FLEXPWM_SET_LDOK, FLEXPWM_SUB0_LDOK | FLEXPWM_SUB1_LDOK | FLEXPWM_SUB2_LDOK)

/***************************************************************************//*!
*
* @brief    Get an input value of GPIO port
*
* @param    SIUL_PAD_x
*
* @return   8bit value
*
* @detail   Read GPIO input value on dedicated pad
*
* @note
*
******************************************************************************/
#define GET_SIUL_INPUT_VAL(param) ioctl(SIUL_BASE,SIUL_READ_INP_VALUE_PAD, param)

/***************************************************************************//*!
*
* @brief    Set GPIO output
*
* @param    SIUL_PAD_x
*
* @return   none
*
* @detail   Set GPIO output value on dedicated pad
*
* @note
*
******************************************************************************/
#define SET_SIUL_OUTPUT(param) ioctl(SIUL_BASE,SIUL_SET_PAD, param)

/***************************************************************************//*!
*
* @brief    Clear GPIO output
*
* @param    SIUL_PAD_x
*
* @return   none
*
* @detail   Clear GPIO output value on dedicated pad
*
* @note
*
******************************************************************************/
#define CLEAR_SIUL_OUTPUT(param) ioctl(SIUL_BASE,SIUL_CLEAR_PAD, param)

/***************************************************************************//*!
*
* @brief    clear CTU interrupt flag register
*
* @param    CTU_ADC_INTFLG - ADC command interrupt flag
*
* @return   none
*
* @detail   The #CTU_CLEAR_INT_FLAGS ioctl command TBD
*
* @note
*
******************************************************************************/
#define CLEAR_CTU_INTERRUPT_FLAG(param)  ioctl(CTU_BASE, CTU_CLEAR_INT_FLAGS, param)


/***************************************************************************//*!
*
* @brief    Set CTU control register
*
* @param    CTU_GENERAL_RELOAD_EN   - General Reload Enable
*
* @return   none
*
* @detail   The #CTU_CONTROL ioctl command TBD
*
* @note
*
******************************************************************************/
#define SET_CTU_CONTROL(param)  ioctl(CTU_BASE, CTU_CONTROL, param)

/***************************************************************************//*!
*
* @brief    Read Flexpwm fault
*
* @param    FLEXPWM_0   - FlexPWM 0 peripheral module
* @param    FLEXPWM_1   - FlexPWM 1 peripheral module   (not available on MPC560xP)
* @param	FLEXPWM_FAULT_1 - fault #1
* @param	FLEXPWM_FAULT_2 - fault #2
* @param	FLEXPWM_FAULT_3 - fault #3
* @param	FLEXPWM_FAULT_4 - fault #4
*
* @return   none
*
* @detail   The #FLEXPWM_GET_FAULT ioctl command TBD
*
* @note
*
******************************************************************************/
#define GET_FLEXPWM_FAULT(param,fault) ioctl(param,FLEXPWM_GET_FAULT,fault)

/***************************************************************************//*!
*
* @brief    Read Flexpwm fault flag
*
* @param    FLEXPWM_0   - FlexPWM 0 peripheral module
* @param    FLEXPWM_1   - FlexPWM 1 peripheral module   (not available on MPC560xP)
* @param	FLEXPWM_FAULT_1 - fault #1
* @param	FLEXPWM_FAULT_2 - fault #2
* @param	FLEXPWM_FAULT_3 - fault #3
* @param	FLEXPWM_FAULT_4 - fault #4
*
* @return   none
*
* @detail   The #FLEXPWM_GET_FAULT_FLAG ioctl command TBD
*
* @note
*
******************************************************************************/
#define GET_FLEXPWM_FAULT_FLAG(param,fault) ioctl(param,FLEXPWM_GET_FAULT_FLAG,fault)

/***************************************************************************//*!
*
* @brief    Get status of CTU error flag register
*
* @param
*
* @return   none
*
* @detail   The #CTU_READ_ERROR_FLAGS ioctl command TBD
*
* @note
*
******************************************************************************/
#define GET_CTU_ERROR_FLAG()	ioctl(CTU_BASE,CTU_READ_ERROR_FLAGS, NULL)

/***************************************************************************//*!
*
* @brief    Set status of CTU error flag register
*
* @param	CTU_MRS_RELOAD_ERROR - Master Reload Signal Reload Error
*
* @return   none
*
* @detail   The #CTU_CLEAR_ERROR_FLAGS ioctl command TBD
*
* @note
*
******************************************************************************/
#define CLEAR_CTU_ERROR_FLAG(param)	ioctl(CTU_BASE,CTU_CLEAR_ERROR_FLAGS, param)


/******************************************************************************
* BSP Exported functions
*******************************************************************************/

#endif /* _MPC5643L_DMCB_BSP_H_ */
