/******************************************************************************
*
* Copyright 2006-2015 Freescale Semiconductor, Inc.
* Copyright 2016-2017 NXP
*
***************************************************************************//*!
*/

#include "derivative.h" /* include peripheral declarations */
#include "sys.h"
#include "MPC5643L_qs.h"
#include "MPC5643L_LLD_qs.h"
#include "state_machine.h"


#include "uart_init.h"
#include "freemaster.h"
#include "freemaster_MPC56xx.h"

#include "mlib.h"
#include "gflib.h"
#include "gmclib.h"
#include "gdflib.h"

#include "fccu.h"

#include "MPC5643L_DMCB_BSP.h"
#include "ADC_MEAS_FLT.h"
#include "POSPE_FLT.h"

#include "MC33937_routines.h"
#include "MC33905_routines.h"

#include "M1_PMSM_appconfig.h"   // from MCAT
#include "M2_PMSM_appconfig.h"   // from MCAT

extern void xcptn_xmpl(void);

/* Routine for code-parts execution time benchmarking*/
#define USED_ET_CHNL_FOR_BENCHMARK          4

#define TIMER_ON \
    ETIMER_0.ENBL.R                                               |= (1<<USED_ET_CHNL_FOR_BENCHMARK);\
    ETIMER_0.CHANNEL[USED_ET_CHNL_FOR_BENCHMARK].CTRL.B.PRISRC     = IPBCLK_DIV_BY_1;\
    ETIMER_0.CHANNEL[USED_ET_CHNL_FOR_BENCHMARK].CNTR.R            = 0x0;\
    ETIMER_0.CHANNEL[USED_ET_CHNL_FOR_BENCHMARK].CTRL.B.CNTMODE    = COUNT_RISING_EDGES_OF_PRIMARY_SRC;

#define TIMER_OFF \
    ETIMER_0.CHANNEL[USED_ET_CHNL_FOR_BENCHMARK].CTRL.B.CNTMODE    = NO_OPERATION;\
    tCLK0 = ETIMER_0.CHANNEL[USED_ET_CHNL_FOR_BENCHMARK].CNTR.R - tOffset; \
    tCLKmax = (tCLK0>tCLKmax)?tCLK0:tCLKmax; \
    tCLKmin = (tCLK0<tCLKmin)?tCLK0:tCLKmin;
/**/

/******************************************************************************
* Defines and macros
******************************************************************************/

/* Select the type of position sensor (Resolver or Encoder) used for FOC.
 * Note: If both are selected then data from both sensors can be displayed in
 * FreeMaster. The particular position/speed feedback used for FOC will be
 * given by switchSensor variable.
 */
#define RESOLVER_FBCK 	1
#define ENCODER_FBCK	1

/* Select the UNI3_2 connector to connect and control second motor.
 * Note: UNI3_1 connector is enabled by default
 */

// enable/disable second motor
#if 1
	#define UNI3_2
#endif

/*
 * Disable and enable interrupt macro
 */
#define DISABLE_IRQ()    asm("wrteei 0")
#define ENABLE_IRQ()     asm("wrteei 1")

/*
 * Declaration macro keeps all necessary settings for single/dual
 * motor control application under one command.
 */
#define DECLARATION(param, uni3_param, pospe_param, cb_param, enc_avaliable, res_avaliable) \
		static pmsmDrive_t  param             = { CFOC_DEFAULT,\
                                                uni3_param##_CFG_DEFAULT,\
                                                pospe_param##_CFG_DEFAULT,\
                                                enc_avaliable,\
                                                res_avaliable,\
                                                MPC5643L_BGA257_##cb_param##_DEFAULT,\
                                                CNTR_STATE_DEFAULT,\
												SCALAR_CNTR_STATE_DEFAULT};

/******************************************************************************
| Typedefs and structures       (scope: module-local) M1_CNTR_STATE_DEFAULT
-----------------------------------------------------------------------------*/

/******************************************************************************
* Local function prototypes
******************************************************************************/

//void main(void);

/* Motor control functions*/
static tBool faultDetection(pmsmDrive_t *ptr);
static tBool focFastLoop(pmsmDrive_t *ptr);
static tBool focSlowLoop(pmsmDrive_t *ptr);
static tBool getMotorControlVariables(pmsmDrive_t *ptr, tFloat I_MAX, tFloat U_MAX);

/******************************************************************************
* Local variables
******************************************************************************/

/* Variables for code execution time  benchmarking*/
static tU16 tCLK0,tCLKmax = 0, tCLKmin = INT32_MAX, tOffset = 30;

/*------------------------------------
 * Application defined modules
 * ----------------------------------*/
static	MC33905_REG_T                sbcDrv = MPC5643L_BGA257_CB_SBC_DEFAULT;

/* DECLARATION macro keeps all necessary settings for single/dual
 * motor control application under one command.
 *
 * @param:
 * 	param       - (M1 / M2)             - M1/M2 is type pmsmDrive_t
 * 	uni3_param  - (UNI3_M1 / UNI3_M2)   - select UNI3 connector.
 * 	pospe_param - (POSPE_M1 / POSPE_M2) - select primary/secondary resolver & encoder connector.
 * 	cb_param    - (CB_M1 / CB_M2)       - select SPI settings for power stage board
 * 	enc_avaliable - (RESOLVER_FBCK / 0) - resolver sensor is available / is NOT
 * 	res_avaliable - (ENCODER_FBCK / 0)  - encoder sensor is available / is NOT
 *
 * @issues:
 * Default setting for pmsmDrive_t structure: 	CFOC_DEFAULT
 * Default setting related to UNI3:				UNI3_M1_CFG_DEFAULT / UNI3_M2_CFG_DEFAULT
 * Default setting related to POSPE:			POSPE_M1_CFG_DEFAULT / POSPE_M2_CFG_DEFAULT
 * Default setting related to preDriver:		MPC5643L_BGA257_CB_M1_DEFAULT / MPC5643L_BGA257_CB_M2_DEFAULT
 */

// declaration for UNI3_1 - default
DECLARATION(M1, UNI3_M1, POSPE_M1, CB_M1, RESOLVER_FBCK, ENCODER_FBCK)	// declaration of the first motor

// declaration for UNI3_2 if enabled
#ifdef UNI3_2
    DECLARATION(M2, UNI3_M2, POSPE_M2, CB_M2, RESOLVER_FBCK, ENCODER_FBCK)	// declaration of the second motor
#endif

/*------------------------------------
 * Currents
 * ----------------------------------*/

/*------------------------------------
 * Voltages
 * ----------------------------------*/

/*------------------------------------
 * Speed/Position
 * ----------------------------------*/

/*------------------------------------
 * SVC-PWM variables
 * ----------------------------------*/

/*------------------------------------
 * FOC variables
 * ----------------------------------*/

/*------------------------------------
 * FREEMASTER SCALES
 * ----------------------------------*/
static volatile tFrac32		m1_fm_speed_w_m    = M1_FM_SPEED_RAD_MEC_SCALE;
static volatile tFrac32		m1_fm_speed_n_e    = M1_FM_SPEED_RPM_EL_SCALE;
static volatile tFrac32		m1_fm_speed_n_m    = M1_FM_SPEED_RPM_MEC_SCALE;
static volatile tFrac32		m1_fm_position_deg = M1_FM_POSITION_DEG_SCALE;

// if second motor defined
#ifdef UNI3_2
static volatile tFrac32		m2_fm_speed_w_m    = M2_FM_SPEED_RAD_MEC_SCALE;
static volatile tFrac32		m2_fm_speed_n_e    = M2_FM_SPEED_RPM_EL_SCALE;
static volatile tFrac32		m2_fm_speed_n_m    = M2_FM_SPEED_RPM_MEC_SCALE;
static volatile tFrac32		m2_fm_position_deg = M2_FM_POSITION_DEG_SCALE;
#endif

/*------------------------------------
 * General use variables
 * ----------------------------------*/
//static tBool			bFcnStatus;
static tBool			centralStop = 0;        	// Switch off both motors
static volatile tU8		btSpeedUp;
static volatile tU8		btSpeedDown;
static volatile tU8		btFlipFlop, btFlipFlopTemp, btFlipFlopHiState, btFlipFlopLoState, SwitchSampleCnt = 0;




int main(void)
{
		if (ME.GS.B.S_CURRENT_MODE == 0x2)
        {
           fccu_Capture();
           fccu_Init();
        }

		// disable interrupts
		DISABLE_IRQ();

		// Initializing of clock, reset, core mode etc.
		sys_init();

		// Configure Interrupts
	    xcptn_xmpl ();

		// Initialize UART
		uart_init();

		// FreeMASTER initialization
		FMSTR_Init();

		// Siu initialize
		siul_init();

		// Initialize DSPI module
		dspi_init();

		// Initialize PIT timer module
		pit_init();

		// Initialize SBC power source
		mc33905Config(&sbcDrv);

		// Initialize FlexPWM modules
		flexpwm0_init();
		flexpwm1_init();

		// Initialize CTU module
		ctu_init();

		// Initialize eTimer0, eTimer1, eTimer2 module
		etimer0_init();
		etimer1_init();

		// Initialize ADC_0 module with HW offset cancellation
		adc0_init();
		adc_off_canc(&ADC_0);

		// Initialize ADC_1 module with HW offset cancellation
		adc1_init();
		adc_off_canc(&ADC_1);

		// MC33937 Configuration with Double RESET Timing logic
		// Since the reset pin is common for both power stages, the configuration
		// is done automatically for both PWSTGs in one step !!!
		mc33937DoubleReset(&(M1.preDrv));

		// Initialize MC33937 driver on 3ph Low voltage power-stage
		mc33937Config(&(M1.preDrv));
		mc33937Enable(&(M1.preDrv));
		mc33937ReadSr(&(M1.preDrv));
		if (M1.preDrv.status.sr0.R!=0)   M1.faultID.B.WrongHardware = 1;

		M1.cntrState.state  						= init;
		M1.cntrState.event   						= e_init;
		M1.cntrState.loadDefSetting					= true;

		// Switch on all FlexPWM channels
		SWITCH_ALL_PWM_ON(M1.uni3PeriphCfg.flexPwmBaseAddr);

		// Enable FlexPWMA/FlexPWMB fault protection
		FAULT_PROTECT_PWMA_ON(M1.uni3PeriphCfg.flexPwmBaseAddr);
		FAULT_PROTECT_PWMB_ON(M1.uni3PeriphCfg.flexPwmBaseAddr);

		// Switch off all FlexPWM channels
		SWITCH_ALL_PWM_OFF(M1.uni3PeriphCfg.flexPwmBaseAddr);

		// if second motor defined
	    #ifdef  UNI3_2
	        // Initialize MC33937 driver on 3ph Low voltage power-stage
	        mc33937Config(&(M2.preDrv));
	        mc33937Enable(&(M2.preDrv));
	        mc33937ReadSr(&(M2.preDrv));
	        if (M2.preDrv.status.sr0.R!=0)   M2.faultID.B.WrongHardware = 1;

	        M2.cntrState.state   						= init;
	        M2.cntrState.event   						= e_init;
			M2.cntrState.loadDefSetting					 = true;

	        // Switch on all FlexPWM channels
	        SWITCH_ALL_PWM_ON(M2.uni3PeriphCfg.flexPwmBaseAddr);

	        // Enable FlexPWMA/FlexPWMB fault protection
	        FAULT_PROTECT_PWMA_ON(M2.uni3PeriphCfg.flexPwmBaseAddr);
	        FAULT_PROTECT_PWMB_ON(M2.uni3PeriphCfg.flexPwmBaseAddr);

	        // Switch off all FlexPWM channels
	        SWITCH_ALL_PWM_OFF(M2.uni3PeriphCfg.flexPwmBaseAddr);
	    #endif


	    if(pCTU->CTUEFR.B.MRS_O)
	       	pCTU->CTUEFR.B.MRS_O = 1;    // CTU MRS_O clearing

	    // Ensure INTC's current priority is 0
	    INTC.CPR_PRC0.B.PRI = 0;
	    // Enable external interrupts
	    ENABLE_IRQ();


	    while(1) {
	   	// Reading faults from MC33937 pre-driver - the same pin for both Power Stages
	   		                if (GET_SIUL_INPUT_VAL(SIUL_PAD_53))
	   		                {
								if (M1.cntrState.readFault)
								{
								mc33937ReadSr(&M1.preDrv);
								if (M1.preDrv.status.sr0.R!=0)   	M1.faultID.B.WrongHardware = 1;
								M1.cntrState.readFault     			= false;
								}

								// if second motor defined
	   		                    #ifdef  UNI3_2
	   		                        if (M2.cntrState.readFault)
	   		                        {
	   		                        mc33937ReadSr(&M2.preDrv);
	   		                        if (M2.preDrv.status.sr0.R!=0)    	M2.faultID.B.WrongHardware = 1;
	   		                        M2.cntrState.readFault     			= false;
	   		                        }
	   		                    #endif
	   		                }
	   	// FreeMaster polling function, shall be executed on background
	  FMSTR_Poll();

	}
}

/***************************************************************************//*!
*
* @brief   FAULT state
*
* @param   pointer to structure of pmsmDrive_t type
*
* @return  none
*
******************************************************************************/
void stateFault(pmsmDrive_t *ptr)
{
    /*-----------------------------------------------------
    Application State Machine - state identification
    ----------------------------------------------------- */
    ptr->cntrState.state   = fault;

    // switch all PWM off
    SWITCH_ALL_PWM_OFF(ptr->uni3PeriphCfg.flexPwmBaseAddr);

    // Disable user application switch
    ptr->cntrState.switchAppOnOff		= false;
    ptr->cntrState.switchAppOnOffState	= false;

    // clear error flag pushing UP/DOWN buttons simultaneously
    if (!btSpeedUp && !btSpeedDown)
    	ptr->cntrState.switchFaultClear = 1;

    if (ptr->cntrState.switchFaultClear)
    {
        // Fault clearing sequence is at the beginning
    	mc33937ClearFault(&ptr->preDrv);
        // Clear Fault PIN flags manually
        CLEAR_FAULT_FLAGS_MAN(ptr->uni3PeriphCfg.flexPwmBaseAddr);
        ptr->faultID.R					= 0x0;      // Clear Fault register
        ptr->faultIDp.R					= 0x0;      // Clear Permanent Fault register

        // Switch used to clear the fault flags when sources of all faults were removed.
        ptr->cntrState.readFault		= true;
        ptr->cntrState.switchFaultClear	= false;

        // set the event = fault cleared
        ptr->cntrState.event			= e_fault_clear;
    }
}

/***************************************************************************//*!
*
* @brief   INIT state
*
* @param   pointer to structure of pmsmDrive_t type
*
* @return  none
*
******************************************************************************/
void stateInit(pmsmDrive_t *ptr)
{
    /*-----------------------------------------------------
    Application State Machine - state identification
    ----------------------------------------------------- */
    tBool InitFcnStatus;

    ptr->cntrState.state                             = init;

    SWITCH_ALL_PWM_OFF(ptr->uni3PeriphCfg.flexPwmBaseAddr);

    /*------------------------------------
     * General use variables
     * ----------------------------------*/
    InitFcnStatus                                    = false;

    /*------------------------------------
     * Application state machine variables
     * ----------------------------------*/
    ptr->cntrState.switchAppOnOff                    = false;
    ptr->cntrState.switchAppOnOffState               = false;
    ptr->cntrState.switchFaultClear                  = false;
    ptr->cntrState.switchAppReset                    = false;



    InitFcnStatus = ADC_ModuleClear(ptr);
    ptr->adc.param.u16CalibSamples                   = 256;    // number of samples = 2^u16CalibSamples = 2^8 = 256

    /*------------------------------------
     * Currents
     * ----------------------------------*/
    ptr->iAbcFbck.fltArg1                = 0.0F;
    ptr->iAbcFbck.fltArg2                = 0.0F;
    ptr->iAbcFbck.fltArg3                = 0.0F;

    ptr->iAlBeFbck.fltArg1               = 0.0F;
    ptr->iAlBeFbck.fltArg2               = 0.0F;

    ptr->iDQReq.fltArg1                  = 0.0F;
    ptr->iDQReq.fltArg2                  = 0.0F;

    ptr->iDQReqZC.fltArg1                = 0.0F;
    ptr->iDQReqZC.fltArg2                = 0.0F;

    ptr->iDQErr.fltArg1                  = 0.0F;
    ptr->iDQErr.fltArg2                  = 0.0F;

    ptr->iDQFbck.fltArg1                 = 0.0F;
    ptr->iDQFbck.fltArg2                 = 0.0F;

    /*------------------------------------
     * Voltages
     * ----------------------------------*/
    ptr->uAlBeReq.fltArg1                = 0.0F;
    ptr->uAlBeReq.fltArg2                = 0.0F;

    ptr->uAlBeReqDCB.fltArg1             = 0.0F;
    ptr->uAlBeReqDCB.fltArg2             = 0.0F;

    ptr->uDQReq.fltArg1                  = 0.0F;
    ptr->uDQReq.fltArg2                  = 0.0F;

    /*------------------------------------
     * Speed/Position
     * ----------------------------------*/
    ptr->thTransform.fltArg1             = 0.0F;
    ptr->thTransform.fltArg2             = 0.0F;

    InitFcnStatus &= POSPE_ModuleClearRes(ptr);
    InitFcnStatus &= POSPE_ModuleClearEnc(ptr);

    // relater to UNI3_1 motor
    if(M1.cntrState.state == init && M1.cntrState.loadDefSetting==true)
    {
		M1.pospeRes.fltResPP                           			= M1_RES_PP_GAIN;
		M1.pospeRes.s32MotorPpScale					   			= M1_MOTOR_PP_GAIN;
		M1.pospeRes.s16MotorPpScaleShift			   			= M1_MOTOR_PP_SHIFT;
		M1.pospeRes.fltMotorPP						   			= M1_MOTOR_PP;

		M1.pospeRes.observer.ATO.pParamInteg.fltC1              = M1_POSPE_RES_TO_INTEG_GAIN;
		M1.pospeRes.observer.ATO.pParamPI.fltCC1sc              = M1_POSPE_RES_TO_CC1;
		M1.pospeRes.observer.ATO.pParamPI.fltCC2sc              = M1_POSPE_RES_TO_CC2;
		M1.pospeRes.observer.ATO.pParamPI.fltUpperLimit         = M1_WEL_MAX;
		M1.pospeRes.observer.ATO.pParamPI.fltLowerLimit         = -M1_WEL_MAX;
		M1.pospeRes.observer.filterMA.fltLambda        			= M1_POSPE_SPEED_FILTER_MA_LAMBDA;

	    M1.pospeEnc.s32MotorPpScale					   			= M1_MOTOR_PP_GAIN;
	    M1.pospeEnc.s16MotorPpScaleShift			   			= M1_MOTOR_PP_SHIFT;
	    M1.pospeEnc.fltMotorPP						   			= M1_MOTOR_PP;

	    M1.pospeEnc.observer.ATO.pParamInteg.fltC1              = M1_POSPE_ENC_TO_INTEG_GAIN;
	    M1.pospeEnc.observer.ATO.pParamPI.fltCC1sc              = M1_POSPE_ENC_TO_CC1;
	    M1.pospeEnc.observer.ATO.pParamPI.fltCC2sc              = M1_POSPE_ENC_TO_CC2;
	    M1.pospeEnc.observer.ATO.pParamPI.fltUpperLimit         = M1_WEL_MAX;
	    M1.pospeEnc.observer.ATO.pParamPI.fltLowerLimit         = -M1_WEL_MAX;
	    M1.pospeEnc.observer.filterMA.fltLambda        			= M1_POSPE_SPEED_FILTER_MA_LAMBDA;

	    M1.scalarControl.wElMax						  			= M1_WEL_MAX;
	    M1.scalarControl.integ.f32C1				   			= M1_SCALAR_INTEG_GAIN;
	    M1.scalarControl.integ.u16NShift 			   			= M1_SCALAR_INTEG_SHIFT;

	    m1_fm_speed_w_m    							   			= M1_FM_SPEED_RAD_MEC_SCALE;
	    m1_fm_speed_n_e    							   			= M1_FM_SPEED_RPM_EL_SCALE;
	    m1_fm_speed_n_m    							   			= M1_FM_SPEED_RPM_MEC_SCALE;
	    m1_fm_position_deg 							   			= M1_FM_POSITION_DEG_SCALE;
    }

    // if second motor defined
	#ifdef UNI3_2
    if(M2.cntrState.state == init && M2.cntrState.loadDefSetting==true)
    {
		M2.pospeRes.fltResPP                           			= M2_RES_PP_GAIN;
		M2.pospeRes.s32MotorPpScale					   			= M2_MOTOR_PP_GAIN;
		M2.pospeRes.s16MotorPpScaleShift			   			= M2_MOTOR_PP_SHIFT;
		M2.pospeRes.fltMotorPP						   			= M2_MOTOR_PP;

		M2.pospeRes.observer.ATO.pParamInteg.fltC1              = M2_POSPE_ENC_TO_INTEG_GAIN;
		M2.pospeRes.observer.ATO.pParamPI.fltCC1sc              = M2_POSPE_ENC_TO_CC1;
		M2.pospeRes.observer.ATO.pParamPI.fltCC2sc              = M2_POSPE_ENC_TO_CC2;
		M2.pospeRes.observer.ATO.pParamPI.fltUpperLimit         = M2_WEL_MAX;
		M2.pospeRes.observer.ATO.pParamPI.fltLowerLimit         = -M2_WEL_MAX;
		M2.pospeRes.observer.filterMA.fltLambda        			= M2_POSPE_SPEED_FILTER_MA_LAMBDA;

	    M2.pospeEnc.s32MotorPpScale					   			= M2_MOTOR_PP_GAIN;
	    M2.pospeEnc.s16MotorPpScaleShift			   			= M2_MOTOR_PP_SHIFT;
	    M2.pospeEnc.fltMotorPP						   			= M2_MOTOR_PP;
	    M2.pospeEnc.observer.ATO.pParamInteg.fltC1              = M2_POSPE_ENC_TO_INTEG_GAIN;
	    M2.pospeEnc.observer.ATO.pParamPI.fltCC1sc              = M2_POSPE_ENC_TO_CC1;
	    M2.pospeEnc.observer.ATO.pParamPI.fltCC2sc              = M2_POSPE_ENC_TO_CC2;
	    M2.pospeEnc.observer.ATO.pParamPI.fltUpperLimit         = M2_WEL_MAX;
	    M2.pospeEnc.observer.ATO.pParamPI.fltLowerLimit         = -M2_WEL_MAX;
	    M2.pospeEnc.observer.filterMA.fltLambda        			= M2_POSPE_SPEED_FILTER_MA_LAMBDA;

	    M2.scalarControl.wElMax						   			= M2_WEL_MAX;
	    M2.scalarControl.integ.f32C1				   			= M2_SCALAR_INTEG_GAIN;
	    M2.scalarControl.integ.u16NShift 			   			= M2_SCALAR_INTEG_SHIFT;

	    m2_fm_speed_w_m    							   			= M2_FM_SPEED_RAD_MEC_SCALE;
	    m2_fm_speed_n_e    							   			= M2_FM_SPEED_RPM_EL_SCALE;
		m2_fm_speed_n_m    							  			= M2_FM_SPEED_RPM_MEC_SCALE;
		m2_fm_position_deg 							   			= M2_FM_POSITION_DEG_SCALE;

    }
	#endif

    ptr->pospeControl.wRotEl						 = 0.0F;
    ptr->pospeControl.wRotElReq					 	 = 0.0F;
    ptr->pospeControl.speedLoopCntr      			 = 0;
    ptr->pospeRes.fltOmegaEl.filt					 = 0.0F;
    ptr->pospeEnc.fltOmegaEl.filt					 = 0.0F;
    ptr->pospeRes.fltThetaEl						 = 0.0F;
    ptr->pospeEnc.fltThetaEl						 = 0.0F;

    /*------------------------------------
     * SVC-PWM variables
     * ----------------------------------*/
    ptr->svmSector                       			 = 1;

    ptr->pwm32.fltArg1                 				 = 0.0001F;
    ptr->pwm32.fltArg2                  			 = 0.00001F;
    ptr->pwm32.fltArg3                  			 = 0.0F;

    ptr->pwm16.s16sub0                  			 = 0.0F;
    ptr->pwm16.s16sub1                  			 = 0.0F;
    ptr->pwm16.s16sub2                  			 = 0.0F;

    /*------------------------------------
     * FOC variables
     * ----------------------------------*/
	ptr->dAxisPI.fltAcc                 			 = 0.0F;
	ptr->dAxisPI.fltInErrK1             			 = 0.0F;
	ptr->qAxisPI.fltAcc                 			 = 0.0F;
	ptr->qAxisPI.fltInErrK1             			 = 0.0F;

	GDFLIB_FilterIIR1Init(&(ptr->dAxisZC));
	GDFLIB_FilterIIR1Init(&(ptr->qAxisZC));

	ptr->speedRamp.fltState             			 = 0.0F;
	ptr->speedPI.fltIntegPartK_1        			 = 0.0F;

	// Related to motor UNI3_1
    if(M1.cntrState.state == init && M1.cntrState.loadDefSetting==true)
    {
    	M1.cntrState.loadDefSetting					 = false;

    	// D-axis PI controller
		M1.dAxisPI.fltCC1sc              			 = M1_D_CC1SC;
		M1.dAxisPI.fltCC2sc               			 = M1_D_CC2SC;
		M1.dAxisPI.fltLowerLimit          			 = MLIB_Mul(-M1_U_DCB_MAX, FLOAT_DIVBY_SQRT3);  //Udc_max/sqrt3;
		M1.dAxisPI.fltUpperLimit          			 = MLIB_Mul(M1_U_DCB_MAX, FLOAT_DIVBY_SQRT3);  //Udc_max/sqrt3;

		// Q-axis PI controller
		M1.qAxisPI.fltCC1sc               			 = M1_Q_CC1SC;
		M1.qAxisPI.fltCC2sc               			 = M1_Q_CC2SC;
		M1.qAxisPI.fltLowerLimit          			 = MLIB_Mul(-M1_U_DCB_MAX, FLOAT_DIVBY_SQRT3);  //Udc_max/sqrt3;
		M1.qAxisPI.fltUpperLimit          			 = MLIB_Mul(M1_U_DCB_MAX, FLOAT_DIVBY_SQRT3);  //Udc_max/sqrt3;

		// D-axis current loop zero cancellation
		M1.dAxisZC.trFiltCoeff.fltB0      			 = M1_D_B0;
		M1.dAxisZC.trFiltCoeff.fltB1      			 = M1_D_B1;
		M1.dAxisZC.trFiltCoeff.fltA1       			 = M1_D_A1;

		// Q-axis current loop zero cancellation
		M1.qAxisZC.trFiltCoeff.fltB0      			 = M1_Q_B0;
		M1.qAxisZC.trFiltCoeff.fltB1      			 = M1_Q_B1;
		M1.qAxisZC.trFiltCoeff.fltA1      			 = M1_Q_A1;

		M1.speedRamp.fltRampUp            			 = M1_SPEED_RAMP_UP;
		M1.speedRamp.fltRampDown          			 = M1_SPEED_RAMP_DOWN;
		M1.scalarControl.wRotReqRamp.fltRampUp       = M1_SCALAR_RAMP_UP;
		M1.scalarControl.wRotReqRamp.fltRampDown	 = M1_SCALAR_RAMP_DOWN;

		M1.speedPI.fltPropGain            			 = M1_SPEED_PI_PROP_GAIN;
		M1.speedPI.fltIntegGain          			 = M1_SPEED_PI_INTEG_GAIN;
		M1.speedPI.fltUpperLimit          			 = M1_SPEED_LOOP_HIGH_LIMIT;
		M1.speedPI.fltLowerLimit          			 = M1_SPEED_LOOP_LOW_LIMIT;

		M1.scalarControl.VHzRatioReq				 = M1_SCALAR_VHZ_FACTOR_GAIN;

		M1.uni3PeriphCfg.alignVoltage				 = M1_ALIGN_VOLTAGE;
    }

    // if second motor defined
	#ifdef UNI3_2
    if(M2.cntrState.state == init && M2.cntrState.loadDefSetting==true)
    {
    	M2.cntrState.loadDefSetting					 = false;

		// D-axis PI controller
		M2.dAxisPI.fltCC1sc              			 = M2_D_CC1SC;
		M2.dAxisPI.fltCC2sc               			 = M2_D_CC2SC;
		M2.dAxisPI.fltLowerLimit          			 = MLIB_Mul(-M2_U_DCB_MAX, FLOAT_DIVBY_SQRT3);
		M2.dAxisPI.fltUpperLimit          			 = MLIB_Mul(M2_U_DCB_MAX, FLOAT_DIVBY_SQRT3);

		// Q-axis PI controller
		M2.qAxisPI.fltCC1sc               			 = M2_Q_CC1SC;
		M2.qAxisPI.fltCC2sc               			 = M2_Q_CC2SC;
		M2.qAxisPI.fltLowerLimit          			 = MLIB_Mul(-M2_U_DCB_MAX, FLOAT_DIVBY_SQRT3);
		M2.qAxisPI.fltUpperLimit          			 = MLIB_Mul(M2_U_DCB_MAX, FLOAT_DIVBY_SQRT3);

		// D-axis current loop zero cancellation
		M2.dAxisZC.trFiltCoeff.fltB0      			 = M2_D_B0;
		M2.dAxisZC.trFiltCoeff.fltB1      			 = M2_D_B1;
		M2.dAxisZC.trFiltCoeff.fltA1       			 = M2_D_A1;

		// Q-axis current loop zero cancellation
		M2.qAxisZC.trFiltCoeff.fltB0      			 = M2_Q_B0;
		M2.qAxisZC.trFiltCoeff.fltB1      			 = M2_Q_B1;
		M2.qAxisZC.trFiltCoeff.fltA1      			 = M2_Q_A1;

		M2.speedRamp.fltRampUp            			 = M2_SPEED_RAMP_UP;
		M2.speedRamp.fltRampDown          			 = M2_SPEED_RAMP_DOWN;
		M2.scalarControl.wRotReqRamp.fltRampUp       = M2_SCALAR_RAMP_UP;
		M2.scalarControl.wRotReqRamp.fltRampDown	 = M2_SCALAR_RAMP_DOWN;

		M2.speedPI.fltPropGain            			 = M2_SPEED_PI_PROP_GAIN;
		M2.speedPI.fltIntegGain          			 = M2_SPEED_PI_INTEG_GAIN;
		M2.speedPI.fltUpperLimit          			 = M2_SPEED_LOOP_HIGH_LIMIT;
		M2.speedPI.fltLowerLimit          			 = M2_SPEED_LOOP_LOW_LIMIT;

		M2.scalarControl.VHzRatioReq				 = M2_SCALAR_VHZ_FACTOR_GAIN;

		M2.uni3PeriphCfg.alignVoltage				 = M2_ALIGN_VOLTAGE;
    }
	#endif

	if(M1.cntrState.state == init)
		{
			M1.uni3PeriphCfg.alignCntr				 = M1_ALIGN_DURATION;
		}

	// if second motor defined
	#ifdef UNI3_2
		if(M2.cntrState.state == init)
		{
			M2.uni3PeriphCfg.alignCntr				 = M2_ALIGN_DURATION;
		}
	#endif

    ptr->scalarControl.UmReq						 = 0.0F;
    ptr->scalarControl.wRotElReq					 = 0.0F;
    ptr->scalarControl.wRotElReqRamp				 = 0.0F;
    ptr->scalarControl.thScalarEl					 = 0;

    ptr->cntrState.ledCounter		 				 = 0;

    // DCBus 1st order filter; Fcut = 100Hz, Ts = 100e-6
    ptr->uDcbFilter.trFiltCoeff.fltB0   			 = M1_UDCB_IIR_B0;
    ptr->uDcbFilter.trFiltCoeff.fltB1   			 = M1_UDCB_IIR_B1;
    ptr->uDcbFilter.trFiltCoeff.fltA1   			 = M1_UDCB_IIR_A1;
    GDFLIB_FilterIIR1Init(&(ptr->uDcbFilter));

    ptr->elimDcbRip.fltModIndex         			 = 0.866025403784439F;
    ptr->elimDcbRip.fltArgDcBusMsr      			 = 0.0F;

    if (!InitFcnStatus)
        {
            ptr->faultID.B.InitError = 1;
            ptr->cntrState.event   = e_fault;
        }
    else
            ptr->cntrState.event   = e_init_done;          // initialization phase finished
}

/***************************************************************************//*!
*
* @brief   READY state
*
* @param   pointer to structure of pmsmDrive_t type
*
* @return  none
*
******************************************************************************/
void stateReady(pmsmDrive_t *ptr)
{
    /*-----------------------------------------------------
    Application State Machine - state identification
    ----------------------------------------------------- */
    ptr->cntrState.state   = ready;
    ptr->cntrState.event   = e_ready;

    // Waiting for user command

    //e.g. load default configuration of control algorithm
    if(ptr->cntrState.loadDefSetting) ptr->cntrState.event   = e_init;
}

/***************************************************************************//*!
*
* @brief   CALIBRATION state - ADC calibration state
*
* @param   pointer to structure of pmsmDrive_t type
*
* @return  none
*
******************************************************************************/
void stateCalib(pmsmDrive_t *ptr)
{
    /*-----------------------------------------------------
    Application State Machine - state identification
    ----------------------------------------------------- */
    volatile tS16        FlexPWM_VAL1_sub0;
    tBool           	 CalibStatus;

    ptr->cntrState.state    = calib;
    ptr->cntrState.event    = e_calib;
    CalibStatus             = false;

    // Calibration sequence is at the beginning
    SWITCH_ALL_PWM_ON(ptr->uni3PeriphCfg.flexPwmBaseAddr);
    CalibStatus = ADC_ModuleCalib(ptr);

    FlexPWM_VAL1_sub0 = GET_FLEXPWM_VAL1_SUB0(ptr->uni3PeriphCfg.flexPwmBaseAddr);

	ptr->pwm16.s16sub0 = MLIB_ShR_F16(FlexPWM_VAL1_sub0,1);
    ptr->pwm16.s16sub1 = MLIB_ShR_F16(FlexPWM_VAL1_sub0,1);
    ptr->pwm16.s16sub2 = MLIB_ShR_F16(FlexPWM_VAL1_sub0,1);

    // Update PWM dutycycles
    SET_3PH_CENTERAL_DUTYCYCLE(ptr->uni3PeriphCfg.flexPwmBaseAddr,ptr->pwm16);
    SET_PWM_LDOK(ptr->uni3PeriphCfg.flexPwmBaseAddr);

    // Test whether the DC offset calibration for one sector ended successfully.
    if (CalibStatus)
    {
        ptr->adc.flag.B.calibInitDone    = 0;
        ptr->adc.flag.B.calibDone        = 0;
        ptr->svmSector                   = MLIB_Add_F32(ptr->svmSector,1);
    }

    // Exit the calibration state when DC calibration is done for all sectors.
    if (ptr->svmSector>6)
    {
        ptr->svmSector                   = 1;

        // Calibration sequence has successfully finished
        ptr->cntrState.event               = e_calib_done;
    }
}

/***************************************************************************//*!
*
* @brief   ALIGNMENT state - motor control d-axes alignment
*
* @param   pointer to structure of pmsmDrive_t type
*
* @return  none
*
******************************************************************************/
void stateAlign(pmsmDrive_t *ptr)
{
    /*-----------------------------------------------------
    Application State Machine - state identification
    ----------------------------------------------------- */
	volatile tS16        FlexPWM_VAL1_sub0;
	volatile tS32		 i;
    tBool           	 AlignStatus;

    ptr->cntrState.state   	 = align;
    ptr->cntrState.event   	 = e_align;

    SWITCH_ALL_PWM_ON(ptr->uni3PeriphCfg.flexPwmBaseAddr);

    // Align sequence is at the beginning
    AlignStatus     		 = false;

    // D-axis voltage is required
    M1.uDQReq.fltArg1        = MLIB_Div(M1.uni3PeriphCfg.alignVoltage,M1_U_DCB_MAX);
    // if second motor defined
	#ifdef UNI3_2
    	M2.uDQReq.fltArg1        = MLIB_Div(M2.uni3PeriphCfg.alignVoltage,M2_U_DCB_MAX);
	#endif

    // Q-Axis voltage is kept at zero
    ptr->uDQReq.fltArg2      = (0.0F);

    // Transformation angle set to 0.
    // Function performs Sin(angle) and Cos (angle), angle = 0;
    GFLIB_SinCos (0.0F, &ptr->thTransform);

    // Transform the dq voltages to AlphaBeta frame
    GMCLIB_ParkInv(&(ptr->uAlBeReq),&(ptr->thTransform),&(ptr->uDQReq));

    ptr->svmSector = GMCLIB_SvmStd(&(ptr->pwm32),&(ptr->uAlBeReq));

    FlexPWM_VAL1_sub0 = GET_FLEXPWM_VAL1_SUB0(ptr->uni3PeriphCfg.flexPwmBaseAddr);

    ptr->pwm16.s16sub0 = (tU16)(MLIB_Mul((tFloat)FlexPWM_VAL1_sub0,ptr->pwm32.fltArg1));
    ptr->pwm16.s16sub1 = (tU16)(MLIB_Mul((tFloat)FlexPWM_VAL1_sub0,ptr->pwm32.fltArg2));
    ptr->pwm16.s16sub2 = (tU16)(MLIB_Mul((tFloat)FlexPWM_VAL1_sub0,ptr->pwm32.fltArg3));

    if (--(ptr->uni3PeriphCfg.alignCntr)<=0)
    {
        ptr->iDQReq.fltArg1 	 = (0.0F);
        ptr->iDQReq.fltArg2 	 = (0.0F);

        ptr->uDQReq.fltArg1 	 = (0.0F);
        ptr->uDQReq.fltArg2 	 = (0.0F);

        ptr->dAxisPI.fltInErrK1  = (0.0F);
        ptr->dAxisPI.fltAcc      = (0.0F);

        ptr->qAxisPI.fltInErrK1  = (0.0F);
        ptr->qAxisPI.fltAcc      = (0.0F);

    	ptr->pwm16.s16sub0 = (tU16)(MLIB_ShR_F16(FlexPWM_VAL1_sub0,1));
        ptr->pwm16.s16sub1 = (tU16)(MLIB_ShR_F16(FlexPWM_VAL1_sub0,1));
        ptr->pwm16.s16sub2 = (tU16)(MLIB_ShR_F16(FlexPWM_VAL1_sub0,1));


        // set initial encoder position to zero
        ptr->pospeEnc.fltThetaEl     	= 0.0F;
        ptr->pospeEnc.fltThetaMe     	= 0.0F;
        ptr->pospeEnc.fltThErr     		= 0.0F;
        AMCLIB_TrackObsrvInit_FLT(&ptr->pospeEnc.observer.ATO);
        AlignStatus = POSPE_SetPositionElEnc(ptr);

      	AlignStatus = POSPE_GetPositionElRes(ptr);
        ptr->pospeRes.fltResOffset   = ptr->pospeRes.fltThetaEl;

        // Alignment sequence status statement
        if (!AlignStatus)   ptr->faultID.B.AlignError	= 1;
        else                ptr->cntrState.event		= e_align_done;
    }

    SET_3PH_CENTERAL_DUTYCYCLE(ptr->uni3PeriphCfg.flexPwmBaseAddr,ptr->pwm16);
    SET_PWM_LDOK(ptr->uni3PeriphCfg.flexPwmBaseAddr);
}

/***************************************************************************//*!
*
* @brief   RUN state
*
* @param   pointer to structure of pmsmDrive_t type
*
* @return  none
*
******************************************************************************/
void stateRun(pmsmDrive_t *ptr)
{
    static tBool stateRunStatus;

    stateRunStatus = false;
    /*-----------------------------------------------------
    Application State Machine - state identification
    ----------------------------------------------------- */
    ptr->cntrState.state   = run;
    ptr->cntrState.event   = e_run;

    // Speed Loop - runs M1_SPEED_LOOP_CNTR -times slower than ADC interrupt frequency
    if (++ptr->pospeControl.speedLoopCntr>=M1_SPEED_LOOP_CNTR)
    {
        ptr->pospeControl.speedLoopCntr    = 0;

        if (!btSpeedUp && ptr->pospeControl.wRotElReq<943.0){
			ptr->pospeControl.wRotElReq = MLIB_Add(ptr->pospeControl.wRotElReq,0.5F);
			// reset ramp state when controlled manually
        	ptr->speedRamp.fltState = ptr->pospeControl.wRotElReq;
        }

        if (!btSpeedDown && ptr->pospeControl.wRotElReq>-943.0){
        	ptr->pospeControl.wRotElReq = MLIB_Sub(ptr->pospeControl.wRotElReq,0.5F);
        	// reset ramp state when controlled manually
        	ptr->speedRamp.fltState = ptr->pospeControl.wRotElReq;
        }

        stateRunStatus  = focSlowLoop(ptr);
        if (!stateRunStatus)       ptr->faultID.B.FOCError = 1;
    }

    // Current loop - runs with ADC interrupt frequency
    stateRunStatus = focFastLoop(ptr);

    if (!stateRunStatus)       ptr->faultID.B.FOCError = 1;

    SET_3PH_CENTERAL_DUTYCYCLE(ptr->uni3PeriphCfg.flexPwmBaseAddr,ptr->pwm16);
    SET_PWM_LDOK(ptr->uni3PeriphCfg.flexPwmBaseAddr);
}

/***************************************************************************//*!
*
* @brief   Fault Detection function
*
* @param   pointer to structure of pmsmDrive_t type
*
* @return  none
*
******************************************************************************/
static tBool faultDetection(pmsmDrive_t *ptr)
{
    tBool faultDetectiontEvent;

    faultDetectiontEvent = false;
    //-----------------------------
    // Actual Faults
    //-----------------------------
    // Fault:   Phase A over-current detected on FAULT PIN FLEXPWM_FAULT_0
    ptr->faultID.B.OverPhaseACurrent    = GET_FLEXPWM_FAULT(ptr->uni3PeriphCfg.flexPwmBaseAddr, FLEXPWM_FAULT_1);


    // Fault:   Phase B over-current detected on FAULT PIN
    ptr->faultID.B.OverPhaseBCurrent    = GET_FLEXPWM_FAULT(ptr->uni3PeriphCfg.flexPwmBaseAddr, FLEXPWM_FAULT_2);

    // Fault:   Phase C over-current detected on FAULT PIN
    ptr->faultID.B.OverPhaseCCurrent    = GET_FLEXPWM_FAULT(ptr->uni3PeriphCfg.flexPwmBaseAddr, FLEXPWM_FAULT_3);

    // Fault:   DC-bus over-voltage
    ptr->faultID.B.OverDCBusVoltage     = GET_FLEXPWM_FAULT(ptr->uni3PeriphCfg.flexPwmBaseAddr, FLEXPWM_FAULT_0);

    // Fault:   DC-bus under-voltage
    M1.faultID.B.UnderDCBusVoltage 	= (M1.adc.measured.fltUdcb.raw < (M1_U_DCB_UNDER)) ? true : false;

    // if second motor defined
    #ifdef UNI3_2
    M2.faultID.B.UnderDCBusVoltage 	= (M2.adc.measured.fltUdcb.raw < (M2_U_DCB_UNDER)) ? true : false;
	#endif

    //-----------------------------
    // Pending Faults
    //-----------------------------

    // Fault:   Phase A over-current detected on FAULT PIN
    ptr->faultIDp.B.OverPhaseACurrent   = GET_FLEXPWM_FAULT_FLAG(ptr->uni3PeriphCfg.flexPwmBaseAddr, FLEXPWM_FAULT_1);

    // Fault:   Phase B over-current detected on FAULT PIN
    ptr->faultIDp.B.OverPhaseBCurrent   = GET_FLEXPWM_FAULT_FLAG(ptr->uni3PeriphCfg.flexPwmBaseAddr, FLEXPWM_FAULT_2);

    // Fault:   Phase C over-current detected on FAULT PIN
    ptr->faultIDp.B.OverPhaseCCurrent   = GET_FLEXPWM_FAULT_FLAG(ptr->uni3PeriphCfg.flexPwmBaseAddr, FLEXPWM_FAULT_3);

    // Fault:   DC-bus over-voltage
    ptr->faultIDp.B.OverDCBusVoltage    = GET_FLEXPWM_FAULT_FLAG(ptr->uni3PeriphCfg.flexPwmBaseAddr, FLEXPWM_FAULT_0);

    if (ptr->faultID.R != 0x0) faultDetectiontEvent = true;
    if (ptr->faultIDp.R != 0x0) faultDetectiontEvent = true;

    return faultDetectiontEvent;
}

/***************************************************************************//*!
*
* @brief   FOC fast (current) loop interrupt service routine
*
* @param   void
*
* @return  none
*
******************************************************************************/
void FOC_Fast_ISR(void)
{
	TIMER_ON

	btSpeedUp			= GET_SIUL_INPUT_VAL(SIUL_PAD_108);
	btSpeedDown			= GET_SIUL_INPUT_VAL(SIUL_PAD_109);
	btFlipFlop			= GET_SIUL_INPUT_VAL(SIUL_PAD_110);

    // User accessible RUN/STOP switch for stopping the application.
    if (btFlipFlop ^ btFlipFlopTemp)
    	{
    		SwitchSampleCnt++;

    		if (btFlipFlop)
    		    btFlipFlopHiState = btFlipFlopHiState + 1;
    		else
    		    btFlipFlopLoState = btFlipFlopLoState + 1;
    	}

    // The routine filters the noise during the On/Off transient on the button
    if (SwitchSampleCnt >= 10)
      {

		  M1.cntrState.switchAppOnOff  = (btFlipFlopHiState > 7) ? false : M1.cntrState.switchAppOnOff;
		  M1.cntrState.switchAppOnOff  = (btFlipFlopLoState > 7) ? true  : M1.cntrState.switchAppOnOff;

		  // if second motor defined
          #ifdef UNI3_2
                  M2.cntrState.switchAppOnOff  = (btFlipFlopHiState > 7) ? false : M2.cntrState.switchAppOnOff;
                  M2.cntrState.switchAppOnOff  = (btFlipFlopLoState > 7) ? true  : M2.cntrState.switchAppOnOff;
          #endif

          btFlipFlopTemp  	= btFlipFlop;
          SwitchSampleCnt 	= 0;
          btFlipFlopHiState = 0;
          btFlipFlopLoState = 0;
      }

    if (centralStop)
    {
        centralStop = false;
        M1.cntrState.switchAppOnOff  = false;

        // if second motor defined
        #ifdef UNI3_2
			M2.cntrState.switchAppOnOff  = false;
        #endif
    }

    // Read motor DCbus voltage, phase currents and speed of the motor
	getMotorControlVariables(&M1, M1_I_MAX, M1_U_DCB_MAX);

	// DCBus resistor braking
	if (M1.adc.measured.fltUdcb.raw > (M1_U_DCB_TRIP))	SET_SIUL_OUTPUT(SIUL_PAD_102);
	else												CLEAR_SIUL_OUTPUT(SIUL_PAD_102);

	// if second motor defined
    #ifdef UNI3_2
		// Read motor DCbus voltage, phase currents and speed of the motor
        getMotorControlVariables(&M2, M2_I_MAX, M2_U_DCB_MAX);

        // DCBus resistor braking
        if (M2.adc.measured.fltUdcb.raw > (M2_U_DCB_TRIP))	SET_SIUL_OUTPUT(SIUL_PAD_126);
        else												CLEAR_SIUL_OUTPUT(SIUL_PAD_126);
    #endif

    // state machine for motor control and LED indication
	state_table[M1.cntrState.event][M1.cntrState.state](&M1);
	state_LED[M1.cntrState.state](&M1, SIUL_PAD_12);

	// if second motor defined
    #ifdef  UNI3_2
		// state machine for motor control and LED indication
        state_table[M2.cntrState.event][M2.cntrState.state](&M2);
        state_LED[M2.cntrState.state](&M2, SIUL_PAD_13);
    #endif

    // Clear ADC command interrupt flag
    CLEAR_CTU_INTERRUPT_FLAG(CTU_ADC_INTFLG);

    // CTU General Reload Enable
    if(pCTU->CTUCR.B.FGRE)   pCTU->CTUCR.B.GRE = 1;

    FMSTR_Recorder();

    TIMER_OFF
}

/***************************************************************************//*!
*
* @brief   Motor control variables measurement
*
* @param   pointer to structure of pmsmDrive_t type
*
* @return  none
*
******************************************************************************/
static tBool getMotorControlVariables(pmsmDrive_t *ptr, tFloat I_MAX, tFloat U_MAX)
{
    static tBool getFcnStatus;

    getFcnStatus    =    false;

    // Phase currents and DCB voltage measurement
	getFcnStatus  = ADC_Measure3Ph(ptr, I_MAX);
	getFcnStatus &= ADC_UdcVoltageMeasure(ptr, U_MAX);

	// Calculate the Tracking observer for both sensors, resolver and encoder
    getFcnStatus &= POSPE_GetPositionElRes(ptr);
    getFcnStatus &= POSPE_GetPositionElEnc(ptr);

    // switchSensor; 0=encoder, 1=resolver
	if (ptr->cntrState.switchSensor)
		ptr->pospeControl.thRotEl = ptr->pospeRes.fltThetaEl;
	else
		ptr->pospeControl.thRotEl = ptr->pospeEnc.fltThetaEl;

	// User accessible switch for stopping the application.
	if (ptr->cntrState.switchAppOnOff ^ ptr->cntrState.switchAppOnOffState)
	{
	 ptr->cntrState.switchAppOnOffState  = ptr->cntrState.switchAppOnOff;
	 ptr->cntrState.event   = (ptr->cntrState.switchAppOnOff) ? e_app_on: e_app_off;
	}

    // Fault detection routine, must be executed prior application state machine
    getFcnStatus &= faultDetection(ptr);
    if (getFcnStatus)    ptr->cntrState.event = e_fault;

    return true;
}

/***************************************************************************//*!
*
* @brief   CTU error interrupt service routine
*
* @param   void
*
* @return  none
*
******************************************************************************/
void CTU_Error_ISR(void)
{
	volatile tU16 temp16;

    M1.faultID.B.CTU_Error = 1;

    #ifdef UNI3_2
        M2.faultID.B.CTU_Error = 1;
    #endif

    //Read CTU error flag status
    temp16 =	GET_CTU_ERROR_FLAG();

    // Clear Mater Reload Signal Reload Error flag
    CLEAR_CTU_ERROR_FLAG(CTU_MRS_RELOAD_ERROR);
}

/***************************************************************************//*!
*
* @brief   Field Oriented Control - slow loop calculations
*
* @param   pointer to structure of pmsmDrive_t type
*
* @return  none
*
******************************************************************************/
static tBool focSlowLoop(pmsmDrive_t *ptr)
{
	POSPE_GetSpeedElRes(ptr);
	POSPE_GetSpeedElEnc(ptr);

	if(ptr->cntrState.switchSensor)	ptr->pospeControl.wRotEl = ptr->pospeRes.fltOmegaEl.filt;
	else							ptr->pospeControl.wRotEl = ptr->pospeEnc.fltOmegaEl.filt;

	ptr->pospeControl.wRotElReqRamp = GFLIB_Ramp(ptr->pospeControl.wRotElReq,&(ptr->speedRamp));
	ptr->pospeControl.wRotElErr     = MLIB_Sub(ptr->pospeControl.wRotElReqRamp,ptr->pospeControl.wRotEl);

    // Speed FO control mode from MCAT
    if(ptr->cntrState.controlMode == 3)
    	ptr->iDQReq.fltArg2			= GFLIB_ControllerPIpAW(ptr->pospeControl.wRotElErr,&(ptr->speedPI));

    return true;
}

/***************************************************************************//*!
*
* @brief   Field Oriented Control - fast loop calculations
*
* @param   void
*
* @return  none
*
******************************************************************************/
static tBool focFastLoop(pmsmDrive_t *ptr)
{
    static volatile tS16		FlexPWM_VAL1_sub0;
    static volatile tS16		FlexPWM_VAL1_sub1;
    static volatile tS16		FlexPWM_VAL1_sub2;
    static volatile tS16		FlexPWM_INIT_sub0;
    static volatile tFrac16		thScalarIntConst;

    // Clarke transform
    GMCLIB_Clark(&ptr->iAlBeFbck,&ptr->iAbcFbck);

    // Scalar (V/f) control mode - open loop regulation
    if(ptr->cntrState.controlMode == 0)
    	{
		ptr->scalarControl.wRotReqRamp.fltRampUp 	= ptr->speedRamp.fltRampUp/10;
		ptr->scalarControl.wRotReqRamp.fltRampDown 	= ptr->speedRamp.fltRampDown/10;

		// generated electric position for scalar control purpose
		ptr->scalarControl.wRotElReqRamp = GFLIB_Ramp(ptr->scalarControl.wRotElReq,&(ptr->scalarControl.wRotReqRamp));
		ptr->scalarControl.thScalarEl  = GFLIB_IntegratorTR_F32(MLIB_ConvertPU_F32FLT(MLIB_Div(ptr->scalarControl.wRotElReqRamp, ptr->scalarControl.wElMax)), &(ptr->scalarControl.integ));
		// Required voltage = VHzRatio * Required Frequency (wRotEl required)
		ptr->scalarControl.UmReq	   = MLIB_Mul(ptr->scalarControl.VHzRatioReq,ptr->scalarControl.wRotElReqRamp);

	    // Function performs Sin(angle) and Cos (angle), angle = MLIB_Mul(MLIB_ConvertPU_FLTF32(ptr->scalarControl.thScalarEl),FLOAT_PI);
	    GFLIB_SinCos (MLIB_Mul(MLIB_ConvertPU_FLTF32(ptr->scalarControl.thScalarEl),FLOAT_PI), &ptr->thTransform);

		ptr->uDQReq.fltArg1          = 0;
		ptr->uDQReq.fltArg2          = ptr->scalarControl.UmReq;
    	}

    // Voltage FO control mode
    if(ptr->cntrState.controlMode == 1 )
    {
		if(ptr->uDQReq.fltArg2==0)
		{
			// if only d-voltage is adjusted
		    // Function performs Sin(angle) and Cos (angle), angle = 0;
		    GFLIB_SinCos (0.0F, &ptr->thTransform);
		}
		else
		{
		    // Function performs Sin(angle) and Cos (angle), angle = ptr->pospeControl.thRotEl;
		    GFLIB_SinCos (ptr->pospeControl.thRotEl, &ptr->thTransform);
		}

		GMCLIB_Park(&ptr->iDQFbck,&ptr->thTransform,&ptr->iAlBeFbck);
    }

    // Current FO control mode and speed FO control mode
    if(ptr->cntrState.controlMode == 2 || ptr->cntrState.controlMode == 3)
    	{
	    // Function performs Sin(angle) and Cos (angle), angle = ptr->pospeControl.thRotEl;
	    GFLIB_SinCos (ptr->pospeControl.thRotEl, &ptr->thTransform);

		GMCLIB_Park(&ptr->iDQFbck,&ptr->thTransform,&ptr->iAlBeFbck);

		ptr->iDQReqZC.fltArg1        = GDFLIB_FilterIIR1(ptr->iDQReq.fltArg1,&(ptr->dAxisZC));
		ptr->iDQReqZC.fltArg2        = GDFLIB_FilterIIR1(ptr->iDQReq.fltArg2,&(ptr->qAxisZC));

		ptr->iDQErr.fltArg1          = MLIB_Sub(ptr->iDQReqZC.fltArg1, ptr->iDQFbck.fltArg1);
		ptr->iDQErr.fltArg2          = MLIB_Sub(ptr->iDQReqZC.fltArg2, ptr->iDQFbck.fltArg2);

		// 95% of available DCbus recalculated to phase voltage = 0.90*uDCB/sqrt(3)
		ptr->dAxisPI.fltUpperLimit   = MLIB_Mul(ptr->adc.measured.fltUdcb.raw,0.51961F);
		ptr->dAxisPI.fltLowerLimit   = MLIB_Neg(ptr->dAxisPI.fltUpperLimit);
		ptr->uDQReq.fltArg1          = GFLIB_ControllerPIrAW(ptr->iDQErr.fltArg1,&ptr->dAxisPI);

		ptr->qAxisPI.fltUpperLimit   = GFLIB_Sqrt(MLIB_Sub(MLIB_Mul(ptr->dAxisPI.fltUpperLimit,ptr->dAxisPI.fltUpperLimit),\
														   MLIB_Mul(ptr->uDQReq.fltArg1,ptr->uDQReq.fltArg1)));
		ptr->qAxisPI.fltLowerLimit   = MLIB_Neg(ptr->qAxisPI.fltUpperLimit);
		ptr->uDQReq.fltArg2          = GFLIB_ControllerPIrAW(ptr->iDQErr.fltArg2,&ptr->qAxisPI);
    	}

    // Park transform
    GMCLIB_ParkInv(&ptr->uAlBeReq,&ptr->thTransform,&ptr->uDQReq);

    // DC bus voltage ripple elimination block
    ptr->elimDcbRip.fltArgDcBusMsr  = ptr->adc.measured.fltUdcb.raw;
    GMCLIB_ElimDcBusRip(&ptr->uAlBeReqDCB,&ptr->uAlBeReq,&ptr->elimDcbRip);

    ptr->svmSector   = GMCLIB_SvmStd(&(ptr->pwm32),&ptr->uAlBeReqDCB);


    FlexPWM_INIT_sub0 = -1*GET_FLEXPWM_INIT_SUB0(ptr->uni3PeriphCfg.flexPwmBaseAddr);

    ptr->pwm16.s16sub0 = (tU16)(MLIB_Mul((tFloat)FlexPWM_INIT_sub0,ptr->pwm32.fltArg1));
    ptr->pwm16.s16sub1 = (tU16)(MLIB_Mul((tFloat)FlexPWM_INIT_sub0,ptr->pwm32.fltArg2));
    ptr->pwm16.s16sub2 = (tU16)(MLIB_Mul((tFloat)FlexPWM_INIT_sub0,ptr->pwm32.fltArg3));

    return (true);
}

/*
 *######################################################################
 *                           End of File
 *######################################################################
*/
