/******************************************************************************
*
* Copyright 2006-2015 Freescale Semiconductor, Inc.
* Copyright 2016-2017 NXP
*
***************************************************************************//*!
*
* @file      motor_structure.h
*
* @author    b34195
* 
* @version   1.0.1.0
* 
* @date      Jul-13-2012
* 
* @brief     Header file for StateMachineFrame "c" project
*
*******************************************************************************
*
* Detailed Description of the file. If not used, remove the separator above.
*
******************************************************************************/

/******************************************************************************
* Includes
******************************************************************************/
#include "SWLIBS_Typedefs.h"

#include "gflib.h"
#include "gmclib.h"
#include "gdflib.h"
#include "MPC5643L_LLD_qs.h"
#include "MC33937_routines.h"
#include "amclib.h"

/******************************************************************************
| Defines and macros            (scope: module-local)
-----------------------------------------------------------------------------*/
#define defaultEncoder 		0 // default sensor
#define defaultResolver 	1 // default sensor

#define ScalarControl   	0 // Scalar control - open loop voltage control
#define VoltageFOControl   	1 // Voltage control loop closed with sensor position
#define CurrentFOControl  	2 // Current control loop closed with sensor position and real currents feedback
#define SpeedFOControl   	3 // Speed control loop closed with sensor position and speed, and real currents feedback

#define CNTR_STATE_DEFAULT  {init, e_init , 0, 0, 0, 0, 0, defaultResolver , 0, 0, 1250, SpeedFOControl}


#define SCALAR_CNTR_STATE_DEFAULT  {0, 0, 0, 0, 0, 0, GFLIB_INTEGRATOR_TR_DEFAULT_F32, GFLIB_RAMP_DEFAULT}

/******************************************************************************
| Typedefs and structures       (scope: module-local)
-----------------------------------------------------------------------------*/

typedef enum {
    init            = 0,
    fault           = 1,
    ready           = 2,
    calib           = 3,
    align           = 4,
    run             = 5
}AppStates;         /* Application state identification user type*/

typedef enum {
    e_fault         = 0,
    e_fault_clear   = 1,
    e_init			= 2,
    e_init_done     = 3,
    e_ready         = 4,
    e_app_on        = 5,
    e_calib         = 6,
    e_calib_done    = 7,
    e_align         = 8,
    e_align_done    = 9,
    e_run           = 10,
    e_app_off       = 11
}AppEvents;         /* Application event identification user type*/

/*! Application fault status user type */
typedef union
{
    volatile tU32 R;
    struct
    {
        volatile tU32 CTU_Error              : 1;   /* Error in CTU hw initialization*/
        volatile tU32 ADC_Error              : 1;   /* Error in ADC hw initialization*/
        volatile tU32 FLEXPWM_Error          : 1;   /* Error in FlexPWM hw initialization */
        volatile tU32 : 4;                          /* RESERVED */
        volatile tU32 InitError              : 1;   /* Error during app initialization */
        volatile tU32 CalibError             : 1;   /* Error during calibration */
        volatile tU32 AlignError             : 1;   /* Error during alignment */
        volatile tU32 FOCError             	 : 1;   /* Error during alignment */
        volatile tU32 : 10;                         /* RESERVED */
        volatile tU32 OffCancError           : 1;   /* Offset Cancellation Error flag */
        volatile tU32 OverPhaseCCurrent      : 1;   /* OverCurrent fault flag */
        volatile tU32 OverPhaseBCurrent      : 1;   /* OverCurrent fault flag */
        volatile tU32 OverPhaseACurrent      : 1;   /* OverCurrent fault flag */
        volatile tU32 OverHeating            : 1;   /* Overheating fault flag */
        volatile tU32 WrongHardware          : 1;   /* Wrong hardware fault flag */
        volatile tU32 MainsFault             : 1;   /* Mains out of range */
        volatile tU32 OverLoad               : 1;   /* Overload Flag */
        volatile tU32 OverDCBusCurrent       : 1;   /* OverCurrent fault flag */
        volatile tU32 UnderDCBusVoltage      : 1;   /* Undervoltage fault flag */
        volatile tU32 OverDCBusVoltage       : 1;   /* Overvoltage fault flag */
    } B;
}AppFaultStatus;    /* Application fault status user type*/

/*------------------------------------------------------------------------*//*!
@brief  Structure containing raw and filtered ADC results
*//*-------------------------------------------------------------------------*/
typedef struct
{
    volatile tFloat    raw;   /*! raw value */
    volatile tFloat    filt;  /*! filtered value */
}adcFiltFlt_t;

/*------------------------------------------------------------------------*//*!
@brief  Structure containing raw and filtered quantities acquired from ADC
		- PMSM Application
*//*-------------------------------------------------------------------------*/
typedef struct
{
	adcFiltFlt_t    fltPhA;     // DC offset measured on phase A current
	adcFiltFlt_t    fltPhB;     // DC offset measured on phase B current
	adcFiltFlt_t    fltPhC;     // DC offset measured on phase C current
	adcFiltFlt_t    fltIdcb;    // DC offset measured on DC bus current
	adcFiltFlt_t    fltUdcb;    // DC offset measured on DC bus voltage
	adcFiltFlt_t    fltTemp;    // DC offset measured on temperature
}adcRawResultPmsm_t;

/*------------------------------------------------------------------------*//*!
@brief  Structure containing raw and filtered quantities acquired from ADC
		- BLDC Application
*//*-------------------------------------------------------------------------*/
typedef struct
{
	adcFiltFlt_t    fltBemfA;    // measured Bemf voltage on phase
	adcFiltFlt_t    fltBemfB;    // measured Bemf voltage on phase
	adcFiltFlt_t    fltBemfC;    // measured Bemf voltage on phase
	adcFiltFlt_t    fltIdcb;     // measured DC-Bus current
	adcFiltFlt_t    fltUdcb;     // measured DC bus voltage
	adcFiltFlt_t    fltTemp;     // measured temperature
}adcRawResultBldc_t;

/*------------------------------------------------------------------------*//*!
@brief  Structure containing variables for software DC offset calibration
		- PMSM Application
*//*-------------------------------------------------------------------------*/
typedef struct
{
    volatile tFloat    fltPhA;	// DC offset measured on phase A current
    volatile tFloat    fltPhB;	// DC offset measured on phase B current
    volatile tFloat    fltPhC;	// DC offset measured on phase C current
    volatile tFloat    fltIdcb;	// DC offset measured on DC bus current
    volatile tFloat    fltUdcb;	// DC offset measured on DC bus voltage
    volatile tFloat    fltTemp;	// DC offset measured on temperature
}adcOffsetPmsm_t;

/*------------------------------------------------------------------------*//*!
@brief  Structure containing variables for software DC offset calibration
		- BLDC Application
*//*-------------------------------------------------------------------------*/
typedef struct
{
    volatile tFloat    fltBemfA;	// DC offset measured on BEMF voltage phase A
    volatile tFloat    fltBemfB;	// DC offset measured on BEMF voltage phase B
    volatile tFloat    fltBemfC;	// DC offset measured on BEMF voltage phase C
    volatile tFloat    fltIdcb;	// DC offset measured on DC bus current
    volatile tFloat    fltUdcb;	// DC offset measured on DC bus voltage
    volatile tFloat    fltTemp;	// DC offset measured on temperature
}adcOffsetBldc_t;

/*------------------------------------------------------------------------*//*!
@brief  Structure containing a variable that stores a number of samples for
        ADC calibration
*//*-------------------------------------------------------------------------*/
typedef struct
{
    tU16     u16CalibSamples; // Number of samples taken for ADC calibration
}adcParam_t;

/*------------------------------------------------------------------------*//*!
@brief  Union containing ADC module operation flags.
*//*-------------------------------------------------------------------------*/
typedef union
{
    volatile tU16 R;
    struct {
        volatile tU16               :14;// RESERVED
        volatile tU16 calibDone     :1; // DC offset calibration done
        volatile tU16 calibInitDone :1; // initial setup for DC offset calibration done
    } B;
}adcFlags_t;

/*------------------------------------------------------------------------*//*!
@brief  ADC module structure containing all ADC related variables - PMSM Application
*//*-------------------------------------------------------------------------*/
typedef struct
{
    volatile adcRawResultPmsm_t  measured;	// raw and filtered ADC results
    volatile adcOffsetPmsm_t     offset[7];	// DC offset measured on ADC channels
    volatile adcParam_t      	 param;		// ADC module calibration parameter
    volatile adcFlags_t      	 flag;		// ADC module operation flags
	tU16 				 	 	 calibCntr;	// Alignment counter variable
}adcModulePmsm_t;

/*------------------------------------------------------------------------*//*!
@brief  ADC module structure containing all ADC related variables - BLDC Application
*//*-------------------------------------------------------------------------*/
typedef struct
{
    volatile adcRawResultBldc_t measured;	// raw and filtered ADC results
    volatile adcOffsetBldc_t    offset[7];	// DC offset measured on ADC channels
    volatile adcParam_t         param;		// ADC module calibration parameter
    volatile adcFlags_t         flag;		// Alignment counter variable
}adcModuleBldc_t;

/*------------------------------------------------------------------------*//*!
@brief  Structure containing moving average filter parameters
*//*-------------------------------------------------------------------------*/
typedef struct
{
    tFrac32 nAcc;
    tU16    nN;
} FILTER_MA2;

/*------------------------------------------------------------------------*//*!
@brief  Structure containing list of base addresses and offsets of particular
		channel related to UNI3 connector and to PMSM application
*//*-------------------------------------------------------------------------*/
typedef struct
{
    tS32   PhABaseAddr;			// Phase A current analog input - ADC Base address
    tS16   PhAChnlOffset;		// Phase A current analog input - ADC channel offset
    tS32   PhBBaseAddr;			// Phase B current analog input - ADC Base address
    tS16   PhBChnlOffset;		// Phase B current analog input - ADC channel offset
    tS32   PhCBaseAddr;			// Phase C current analog input - ADC Base address
    tS16   PhCChnlOffset;		// Phase C current analog input - ADC channel offset
    tS32   UdcBaseAddr;			// Udc bus voltage analog input - ADC Base address
    tS16   UdcChnlOffset;		// Udc bus voltage analog input - ADC channel offset
    tU16   alignCntr;			// Alignment counter variable
    tFloat alignVoltage;
    tS32   flexPwmBaseAddr;		// flexPWM Base address related to UNI3 connector
}uni3PeriphCfgPmsm_t;

/*------------------------------------------------------------------------*//*!
@brief  Structure containing list of base addresses and offsets of particular
		channel related to UNI3 connector and to BLDC application
*//*-------------------------------------------------------------------------*/
typedef struct
{
    tS32   bemfABaseAddr;       // BEMF A analog input Base address
    tS16   bemfAChnlOffset;     // BEMF A analog input channel address offset
    tS32   bemfBBaseAddr;       // BEMF B analog input Base address
    tS16   bemfBChnlOffset;     // BEMF B analog input channel address offset
    tS32   bemfCBaseAddr;       // BEMF C analog input Base address
    tS16   bemfCChnlOffset;     // BEMF C analog input channel address offset
    tS32   udcBaseAddr;         // Udcb analog input Base address
    tS16   udcChnlOffset;       // Udcb analog input channel address offset
    tS32   idcBaseAddr;         // Idcb analog input Base address
    tS16   idcChnlOffset;       // Idcb analog input channel address offset
    tU16   alignCntr;           // Alignment counter variable
    tS32   flexPwmBaseAddr;     // flexPWM Base address related to UNI3 connector
}uni3PeriphCfgBldc_t;

/*------------------------------------------------------------------------*//*!
@brief  Structure containing raw ADC results
*//*-------------------------------------------------------------------------*/
typedef struct
{
    AppStates       state;                  // raw value
    AppEvents       event;                  // raw value
    tBool           switchAppOnOff;         // raw value
    tBool           switchAppOnOffState;    // raw value
    tBool           switchFaultClear;       // raw value
    tBool           switchAppReset;         // raw value
    tBool			loadDefSetting;			// load default application settings
    tBool           switchSensor;   		// switch between encoder/resolver sensor
    tBool           readFault;				// Read fault status
    tS16            ledCounter;				// counter for led toggling
    tS16            ledFlashing;			// defines flashing period
    tS16			controlMode;			// defines the control mode within a control structure; range(0 - 3)
}driveStates_t;

/*------------------------------------------------------------------------*//*!
@brief  Structure containing raw POSPE raw/filtered values
*//*-------------------------------------------------------------------------*/
typedef struct
{
	volatile tFloat	raw;                    // raw value
	volatile tFloat	filt;                   // filtered value
}pospeFiltFlt_t;

/*------------------------------------------------------------------------*//*!
@brief  Structure containing POSPE ATO observer variables
*//*-------------------------------------------------------------------------*/
//povodne
/*typedef struct
{
    GFLIB_CONTROLLER_PIAW_R_T   pi;          // PI recurrent controller
    GFLIB_INTEGRATOR_TR_T       integ;		 // Integrator settings
    GDFLIB_FILTER_MA_T          filterMA;	 // Moving average filter setting
}pospeObserver_t;*/

//nove

typedef struct
{
	AMCLIB_TRACK_OBSRV_T_FLT        ATO;// Angle tracking observer
    GDFLIB_FILTER_MA_T          filterMA;	 // Moving average filter setting
}pospeObserver_t;

/*------------------------------------------------------------------------*//*!
@brief  Structure containing ADC channels configuration related to resolver
*//*-------------------------------------------------------------------------*/
typedef struct
{
    tS32   SinBaseAddr;			// ADC base address for "sin" measurement
    tS16   SinChnlOffset;		// ADC channel address for "sin" measurement
    tS32   CosBaseAddr;			// ADC base address for "cos" measurement
    tS16   CosChnlOffset;		// ADC channel address for "cos" measurement
}resCfg_t;

/*------------------------------------------------------------------------*//*!
@brief  Structure containing eTimer configuration related to resolver
*//*-------------------------------------------------------------------------*/
typedef struct
{
	tS32   eTimerAddr;			// eTimer base address
}encCfg_t;

/*------------------------------------------------------------------------*//*!
@brief  Structure containing Resolver/Encoder settings
*//*-------------------------------------------------------------------------*/
typedef struct
{
    resCfg_t    resCfg;			// resolver configuration
    encCfg_t    encCfg;			// encoder configuration
    tBool		resAvailable;	// resolver availability
    tBool		encAvailable;	// encoder availability
}pospePeriphCfg_t;

/*------------------------------------------------------------------------*//*!
@brief  Structure containing position/speed module variables
*//*-------------------------------------------------------------------------*/
typedef struct
{
	tFrac32		thScalarEl;		// El. position entering the scalar control
    tFloat		wRotElReq;		// Required frequency [in rad/s] of supply voltage for scalar control
    tFloat		wRotElReqRamp;  // Required frequency [in rad/s] as a ramp
    tFloat		wElMax; 			// Required frequency as a ramp
    tFloat		UmReq;			// Required magnitude of supply voltage for scalar control
    tFloat		VHzRatioReq;	// V/f ratio
    GFLIB_INTEGRATOR_TR_T_F32       integ;		 // F32 integrator -> position wrapping around +-1
    GFLIB_RAMP_T				wRotReqRamp;
} scalarControl_t;

/*------------------------------------------------------------------------*//*!
@brief  Structure containing position/speed module variables
*//*-------------------------------------------------------------------------*/
typedef struct
{
    tFloat		thRotEl;		// El. position entering the control loop
    tFloat		wRotEl;			// El. speed entering to the control loop
    tFloat		wRotElReq;		// Required el. speed
    tFloat		wRotElReqRamp;	// Required el. speed converted to the ramp shape
    tFloat		wRotElErr;		// Error of the el. speed entering to speed controller
    tS16		speedLoopCntr;	// rate between speed and current loop
} pospeControl_t;

/*------------------------------------------------------------------------*//*!
@brief  Structure containing position/speed module variables
*//*-------------------------------------------------------------------------*/
typedef struct
{
    tFloat			fltThErr;
    tFloat			fltThetaEl;                 // Electrical position in step k
    tFloat			fltThetaMe;                 // Mechanical position in step k
    tFloat			fltThetaElRes;              // Electrical position in step k related to resolver

    tFloat			fltResOffset;               // Offset of resolver from absolute zero position
    tFloat			fltResSin;                  // Resolver sin
    tFloat			fltResCos;				    // Resolver cos

    tFrac32			s32MotorPpScale;            // Scale of motor pole pairs - fixed point interpretation
    tS16			s16MotorPpScaleShift;       // Scale of motor pole pairs - fixed point interpretation
    tFloat			fltMotorPP;           	    // Floating point interpretation of Motor PP
    tFloat			fltResPP;                   // Gain of resolver electrical position to mechanical - "shaft" position

    pospeFiltFlt_t	fltOmegaEl;                 // Electrical angular velocity of motor
    pospeFiltFlt_t	fltOmegaElRes;              // Electrical angular velocity related to resolver
    pospeFiltFlt_t	fltOmegaMe;                 // Mechanical angular velocity of motor
    pospeObserver_t	observer;					// Position/Speed observer parameters
} pospeModule_t;

/*------------------------------------------------------------------------*//*!
@brief  Structure containing list of all variables related to FOC PMSM control
*//*-------------------------------------------------------------------------*/
typedef struct{
    AppFaultStatus				faultID;        // Application faults
    AppFaultStatus				faultIDp;       // Application fault flags
    tU32						svmSector;      // Space Vector Modulation sector
    SWLIBS_2Syst				iDQFbck;        // dq - axis current feedback
    SWLIBS_2Syst				iAlBeFbck;      // Alpha/Beta - axis current feedback
    SWLIBS_2Syst				iDQReq;         // dq - axis required currents, given by speed PI
    SWLIBS_2Syst				uDQReq;         // dq - axis required voltages given by current PIs
    SWLIBS_2Syst				iDQReqZC;       // Transfer function zeros cancellation in current branch
    SWLIBS_2Syst				iDQErr;         // Error between the reference and feedback signal
    SWLIBS_2Syst				uAlBeReq;       // Alpha/Beta required voltages
    SWLIBS_2Syst				uAlBeReqDCB;    // Alpha/Beta required voltages after DC Bus ripple elimination
    SWLIBS_2Syst				thTransform;    // Transformation angle - enters to Park transformation
    SWLIBS_3Syst				iAbcFbck;       // Three phases current feedback
    SWLIBS_3Syst				pwm32;          // Three phase 32bit Duty-Cycles estimated from uAlBeReqDCB
    FLEXPWM_VAL_COMPL			pwm16;          // Three phase 16bit Duty-Cycles feeding PWM registers
    GMCLIB_ELIMDCBUSRIP_T		elimDcbRip;     // Predefined structure related to DC Bus voltage ripple elimination
    GFLIB_CONTROLLER_PIAW_R_T	dAxisPI;        // Predefined structure related to d-axis current PI controller
    GFLIB_CONTROLLER_PIAW_R_T	qAxisPI;        // Predefined structure related to q-axis current PI controller
    GFLIB_CONTROLLER_PIAW_P_T	speedPI;        // Predefined structure related to Speed PI controller
    GFLIB_RAMP_T				speedRamp;      // Reference speed ramp generation
    GDFLIB_FILTER_IIR1_T		dAxisZC;        // Predefined structure related to d-axis current zero cancellation - filter parameter
    GDFLIB_FILTER_IIR1_T		qAxisZC;        // Predefined structure related to q-axis current zero cancellation - filter parameter
    GDFLIB_FILTER_IIR1_T		uDcbFilter;     // DC bus voltage filter settings
    adcModulePmsm_t				adc;            // ADC measurement
    pospeModule_t				pospeRes;       // Position/Speed module for resolver
    pospeModule_t				pospeEnc;       // Position/Speed module for encoder
    pospeControl_t				pospeControl;   // Position/Speed variables needed for control
    uni3PeriphCfgPmsm_t			uni3PeriphCfg;  // ADC measurement channels configuration
    pospePeriphCfg_t			pospeCfg;       // Encoder / Resolver configuration
    MC33937_SET_T				preDrv;         // PreDriver configuration
    driveStates_t				cntrState;		// Control states variables
    scalarControl_t				scalarControl;		// Control states variables
}pmsmDrive_t;

typedef struct
{
    tFrac32 K1;
    tFrac32 out_last;
}rlInvParamFilter1ord;

/*------------------------------------------------------------------------*//*!
@brief  Structure containing list of all variables related to Six Step BLDC control
*//*-------------------------------------------------------------------------*/
typedef struct{
    AppFaultStatus				faultID;        // Application faults
    AppFaultStatus				faultIDp;       // Application fault flags
    tU32						svmSector;      // Space Vector Modulation sector
    SWLIBS_2Syst				iDQFbck;        // dq - axis current feedback
    SWLIBS_2Syst				iAlBeFbck;      // Alpha/Beta - axis current feedback
    SWLIBS_2Syst				iDQReq;         // dq - axis required currents, given by speed PI
    SWLIBS_2Syst				uDQReq;         // dq - axis required voltages given by current PIs
    SWLIBS_2Syst				iDQReqZC;       // Transfer function zeros cancellation in current branch
    SWLIBS_2Syst				iDQErr;         // Error between the reference and feedback signal
    SWLIBS_2Syst				uAlBeReq;       // Required dq currents transformed into the Alpha/Beta orth. system
    SWLIBS_2Syst				uAlBeReqDCB;    // Alpha/Beta required voltages after DC Bus ripple elimination
    SWLIBS_2Syst				thTransform;    // Transformation angle
    SWLIBS_3Syst				iAbcFbck;       // Three phases current feedback
    SWLIBS_3Syst				pwm32;          // Three phase 32bit Duty-Cycles estimated from uAlBeReqDCB
    FLEXPWM_VAL_COMPL			pwm16;          // Three phase 16bit Duty-Cycles feeding PWM registers
    GMCLIB_ELIMDCBUSRIP_T		elimDcbRip;     // Predefined structure related to DC Bus voltage ripple elimination
    GFLIB_CONTROLLER_PIAW_R_T	dAxisPI;        // d-axis current PI controller
    GFLIB_CONTROLLER_PIAW_R_T	qAxisPI;        // q-axis current PI controller
    GFLIB_CONTROLLER_PIAW_P_T	speedPI;        // Speed PI controller
    GFLIB_RAMP_T				speedRamp;      // Reference speed ramp generation
    GDFLIB_FILTER_IIR1_T		dAxisZC;        // d-axis current zero cancellation - filter parameter
    GDFLIB_FILTER_IIR1_T		qAxisZC;        // q-axis current zero cancellation - filter parameter
    GDFLIB_FILTER_IIR1_T		uDcbFilter;     // DC bus voltage filter settings
    adcModuleBldc_t				adc;            // ADC measurement
    pospeModule_t				pospeRes;       // Position/Speed module for resolver
    pospeModule_t				pospeEnc;       // Position/Speed module for encoder
    pospeControl_t				pospeControl;   // Position/Speed variables needed for control
    uni3PeriphCfgBldc_t			uni3PeriphCfg;  // ADC measurement channels configuration
    pospePeriphCfg_t			pospeCfg;       // Encoder / Resolver configuration
    MC33937_SET_T				preDrv;         // PreDriver configuration
    driveStates_t				cntrState;		// Control states variables
}bldcDrive_t;


